SystemOrganization addCategory: #'GSoC-SpeechBubble'!
SystemOrganization addCategory: #'GSoC-Graphing'!
SystemOrganization addCategory: #'GSoC-Utils'!
SystemOrganization addCategory: #'GSoC-IO'!

TextMorph subclass: #GSoCTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Utils'!

!GSoCTextMorph commentStamp: '<historical>' prior: 0!
TextMorph that changes the focus to the next object when pressing <Enter>. It also changes the border color when has focus.
I know a whole new class for these purposes only it's ugly but I don't know how to do this easily using just TextMorph.!

!GSoCTextMorph methodsFor: 'as yet unclassified' stamp: 'Richo 5/16/2010 16:21'!
handleKeystroke: anEvent
	"System level event handling."

	| pasteUp |
	anEvent wasHandled ifTrue:[^self].
	(self handlesKeyboard: anEvent) ifFalse:	[^ self].
	anEvent wasHandled: true.
	((anEvent keyCharacter = Character tab) or: [anEvent keyCharacter = Character cr]) ifTrue:
		["Allow passing through text morph inside pasteups"
		(self wouldAcceptKeyboardFocusUponTab and:
				[(pasteUp _ self pasteUpMorphHandlingTabAmongFields) notNil])
			ifTrue:[^ pasteUp tabHitWithEvent: anEvent]].
	self keyStroke: anEvent! !

!GSoCTextMorph methodsFor: 'as yet unclassified' stamp: 'Richo 5/16/2010 16:20'!
initialize
super initialize.! !

!GSoCTextMorph methodsFor: 'as yet unclassified' stamp: 'Richo 5/16/2010 16:29'!
keyboardFocusChange: aBoolean 
super keyboardFocusChange: aBoolean.
aBoolean ifTrue: [self borderColor: Color red]  ifFalse: [self borderColor: Color black]! !

!GSoCTextMorph methodsFor: 'as yet unclassified' stamp: 'Richo 6/17/2010 23:45'!
setExtentFromHalo: aPoint
	super setExtentFromHalo: aPoint.
	(owner respondsTo: #notifyExtentChange:) ifTrue: [owner notifyExtentChange: self]! !

Object subclass: #SyntaxErrorOmission
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Utils'!

!SyntaxErrorOmission commentStamp: 'Richo 5/16/2010 13:41' prior: 0!
This object is only used to avoid a SyntaxError dialog when doing Compiler>>#evaluate:notifying:logged:. It simply omits the message.
The Compiler stuff is done in the table morphs to get a number from the text. I know it should be easy to simply sent #asNumber to the string, but I did it this way because it allows to write stuff like:

'Float pi'
'(11/9)'
'15 * 6'!

!SyntaxErrorOmission methodsFor: 'as yet unclassified' stamp: 'Richo 5/16/2010 13:27'!
notify: aByteString at: aSmallInteger in: aReadStream 
! !

!SyntaxErrorOmission methodsFor: 'as yet unclassified' stamp: 'Richo 5/16/2010 13:27'!
selectionInterval
^(0 to: 0)! !

!SyntaxErrorOmission methodsFor: 'as yet unclassified' stamp: 'Richo 5/16/2010 13:29'!
text
^Text new! !

Object subclass: #Vector
	instanceVariableNames: 'initialPoint terminalPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!Vector commentStamp: 'Richo 4/18/2010 00:23' prior: 0!
This class represents a Vector. It lets you define initial point, terminal point, magnitude, and angle. Every point is inmutable.

To create a Vector you have the following protocol: 
	Vector angle: 45 magnitude: 5.
	Vector terminalPoint: 10@10
	Vector initialPoint: 5@5 angle: 45 magnitude: 5.
	Vector initialPoint: 5@5 terminalPoint: 10@10.

If the initial point is not specified it uses 0@0. It is also possible to create a Vector using the Point's special message #to:
	((5@5) to: (10@10))!

!Vector class methodsFor: 'instance creation' stamp: 'Richo 4/12/2010 22:27'!
angle: degrees magnitude: magnitude
^self initialPoint: 0@0 angle: degrees magnitude: magnitude! !

!Vector class methodsFor: 'instance creation' stamp: 'Richo 4/12/2010 22:22'!
initialPoint: initialPoint angle: degrees magnitude: magnitude
| o a |
o := degrees degreesToRadians sin * magnitude.
a := degrees degreesToRadians cos * magnitude.
^self initialPoint: initialPoint terminalPoint: (initialPoint + (a@o))! !

!Vector class methodsFor: 'instance creation' stamp: 'Richo 4/12/2010 16:21'!
initialPoint: initialPoint terminalPoint: terminalPoint
	^(self basicNew setInitialPoint: initialPoint terminalPoint: terminalPoint) initialize! !

!Vector class methodsFor: 'instance creation' stamp: 'Richo 4/12/2010 16:30'!
terminalPoint: aPoint 
^self initialPoint: 0@0 terminalPoint: aPoint! !

!Vector methodsFor: 'accessing' stamp: 'Richo 4/13/2010 08:48'!
angle
"Returns an angle in degrees from 0 to 360"
| angle diff | 
diff := terminalPoint - initialPoint.
diff x = 0 ifTrue: [diff y >= 0 ifTrue: [^90] ifFalse: [^270]].
angle := (diff y abs / diff x abs) arcTan radiansToDegrees.
diff x > 0 ifTrue: [
	diff y >= 0 ifFalse: [angle := 360 - angle]
]
ifFalse: [
	diff y >= 0 ifTrue: [angle := 180 - angle] ifFalse: [angle := 180 + angle]
].
^angle! !

!Vector methodsFor: 'comparing' stamp: 'Richo 4/13/2010 08:59'!
hash
^initialPoint hash bitXor: terminalPoint hash! !

!Vector methodsFor: 'accessing' stamp: 'Richo 4/12/2010 16:25'!
initialPoint
^initialPoint! !

!Vector methodsFor: 'accessing' stamp: 'Richo 4/12/2010 16:28'!
magnitude
| diff |
diff := terminalPoint - initialPoint.
^(diff x squared + diff y squared) sqrt! !

!Vector methodsFor: 'printing' stamp: 'Richo 4/13/2010 08:46'!
printOn: aStream
aStream nextPut: $(.
initialPoint printOn: aStream.
aStream nextPut: $).
aStream nextPutAll: ' to: '.
aStream nextPut: $(.
terminalPoint printOn: aStream.
aStream nextPut: $).
! !

!Vector methodsFor: 'private' stamp: 'Richo 4/12/2010 16:22'!
setInitialPoint: aPoint terminalPoint: anotherPoint
initialPoint := aPoint.
terminalPoint := anotherPoint! !

!Vector methodsFor: 'accessing' stamp: 'Richo 4/12/2010 16:25'!
terminalPoint
^terminalPoint ! !

!Vector methodsFor: 'comparing' stamp: 'Richo 4/13/2010 08:50'!
= anotherObject
(anotherObject isKindOf: self class)
	ifFalse: [^ false].
	^initialPoint = anotherObject initialPoint and: [terminalPoint = anotherObject terminalPoint]! !

RectangleMorph subclass: #BarMorph
	instanceVariableNames: 'graph bar index labelMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!BarMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/13/2010 20:19'!
additionsToViewerCategories
^ #(
(#plot (
#(slot barLabel '' String readWrite Player getBarLabel Player setBarLabel:  )
#(slot barValue '' Number readWrite Player getBarValue Player setBarValue:  )
#(slot barColor '' Color readWrite Player getBarColor Player setBarColor:  )
#(command remove '' )
))

)! !

!BarMorph class methodsFor: 'instance creation' stamp: 'Richo 4/24/2010 14:48'!
graph: aBarGraphMorph bar: anAssociation index: aNumber
^(self basicNew setGraph: aBarGraphMorph bar: anAssociation index: aNumber) initialize! !

!BarMorph methodsFor: 'accessing' stamp: 'Richo 4/24/2010 09:38'!
bar
^bar! !

!BarMorph methodsFor: 'accessing' stamp: 'Richo 5/21/2010 17:47'!
bar: anAssociation 
	self privateExtent: (graph extentFor: anAssociation);
		privateCenter: (graph centerFor: anAssociation index: index);
		updateLabel: anAssociation.
	bar := anAssociation ! !

!BarMorph methodsFor: 'initialize-release' stamp: 'Richo 4/27/2010 14:47'!
dismissViaHalo
	self remove.
	super dismissViaHalo! !

!BarMorph methodsFor: 'accessing' stamp: 'Richo 4/27/2010 15:13'!
extent: aPoint
super extent: aPoint.
owner notNil ifTrue: [self update].
! !

!BarMorph methodsFor: 'initialize-release' stamp: 'Richo 4/27/2010 15:09'!
initialize
super initialize.
self width: 30; color: Color blue muchDarker.
labelMorph := StringMorph new.
self update! !

!BarMorph methodsFor: 'initialize-release' stamp: 'Richo 4/24/2010 22:54'!
newPlayerInstance
^BarPlayer newUserInstance! !

!BarMorph methodsFor: 'accessing' stamp: 'Richo 4/27/2010 15:16'!
position: aPoint
super position: aPoint.
self update! !

!BarMorph methodsFor: 'private' stamp: 'Richo 4/27/2010 15:15'!
privateCenter: aPoint
self privatePosition: (aPoint - (self extent // 2))! !

!BarMorph methodsFor: 'private' stamp: 'Richo 4/27/2010 15:12'!
privateExtent: aPoint
super extent: aPoint! !

!BarMorph methodsFor: 'private' stamp: 'Richo 4/27/2010 15:15'!
privatePosition: aPoint
super position: aPoint! !

!BarMorph methodsFor: 'initialize-release' stamp: 'Richo 4/27/2010 14:48'!
remove
	graph removeBar: bar! !

!BarMorph methodsFor: 'private' stamp: 'Richo 4/24/2010 14:48'!
setGraph: aBarGraphMorph bar: anAssociation index: aNumber
graph := aBarGraphMorph.
bar := anAssociation.
index := aNumber! !

!BarMorph methodsFor: 'initialize-release' stamp: 'Richo 5/13/2010 20:36'!
update
self update: index! !

!BarMorph methodsFor: 'accessing' stamp: 'Richo 5/21/2010 18:09'!
updateLabel: anAssociation
	| label value |
	label := anAssociation key.
	value := anAssociation value.
	labelMorph contents: label.
	graph isVerticalGraph
		ifTrue: [value >= 0
				ifTrue: [labelMorph center: self bottomCenter + (0 @ 10)]
				ifFalse: [labelMorph center: self topCenter - (0 @ 10)]]
		ifFalse: [value <= 0
				ifTrue: [labelMorph center: self rightCenter + ((labelMorph width / 2) rounded + 3 @ 0)]
				ifFalse: [labelMorph center: self leftCenter - ((labelMorph width / 2) rounded + 3 @ 0).
						labelMorph left <= graph left
							ifTrue: [labelMorph left: self right + 3]]].
	self addMorph: labelMorph.! !

!BarMorph methodsFor: 'initialize-release' stamp: 'Richo 5/13/2010 20:35'!
update: newIndex
index := newIndex.
self penUpWhile: [self bar: bar].
! !

AlignmentMorph subclass: #BarTableMorph
	instanceVariableNames: 'numberOfPoints graph list scrollPane numberOfBars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!BarTableMorph class methodsFor: 'viewer categories' stamp: 'Richo 4/27/2010 12:50'!
additionsToViewerCategories
^ #(
(#plot (
#(command apply '' )
#(command addBar '' )
#(command removeBar '' )
#(slot barCount '' Number readOnly Player getBarCount Player unused )
#(slot barIndex '' Number readWrite Player getBarIndex Player setBarIndex:  )
#(slot barValue '' Number readWrite Player getBarValue Player setBarValue:  )
#(slot barLabel '' String readWrite Player getBarLabel Player setBarLabel:  )
#(slot barsFilled '' Number readOnly Player getBarsFilled Player unused )
))

)! !

!BarTableMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/26/2010 23:08'!
includeInNewMorphMenu
	"Return true for all classes that can be instantiated from the menu"
	^ false! !

!BarTableMorph class methodsFor: 'instance creation' stamp: 'Richo 4/26/2010 23:08'!
on: aDataGraphMorph
^(self basicNew setGraph: aDataGraphMorph) initialize ! !

!BarTableMorph methodsFor: 'actions' stamp: 'Richo 4/26/2010 23:13'!
addBar
	| |
	numberOfBars := numberOfBars + 1.
	list addMorph: (self makeRow: nil)   asElementNumber: numberOfBars + 1.
	self updateScrollPane.! !

!BarTableMorph methodsFor: 'actions' stamp: 'Richo 4/27/2010 12:36'!
apply
| newBars oldBar newBar |
newBars := self bars.
1 to: newBars size do: [:i|
	oldBar := graph barAt: i ifAbsent: [nil].
	newBar := newBars at: i.
	oldBar isNil
	ifTrue: [graph addBar: newBar]
	ifFalse: [graph barAt: i put: newBar]
	].
newBars size < graph barCount
	ifTrue: [graph removeBarsFrom: newBars size + 1 to: graph barCount].! !

!BarTableMorph methodsFor: 'accessing' stamp: 'Richo 5/16/2010 16:42'!
bars
"Searches across the submorphs and returns an array of points with the data written by the user"
	| data firstSubmorph secondSubmorph label value |
	data := OrderedCollection new.
	list
		submorphsDo: [:morph | morph isAlignmentMorph
				ifTrue: [
					label := value := nil.
					firstSubmorph := morph submorphs first.
					secondSubmorph := morph submorphs second.
					firstSubmorph isTextMorph
						ifTrue: [firstSubmorph contents asString = ''
								ifFalse: [label := firstSubmorph contents asString]].
					secondSubmorph isTextMorph
						ifTrue: [secondSubmorph contents asString = ''
								ifFalse: [value := self valueFromString: secondSubmorph contents asString]].
					(label notNil
							and: [value isNumber])
						ifTrue: [data add: label -> value]]].
	^ data! !

!BarTableMorph methodsFor: 'initialize' stamp: 'Richo 4/26/2010 23:08'!
initialize
	| data |
	super initialize.
	self borderWidth: 0;
		
		color: ((Color
				r: 0.817
				g: 0.864
				b: 0.797)
				mixed: 0.75
				with: Color blue muchDarker);
		 hResizing: #shrinkWrap;
		 vResizing: #shrinkWrap;
		 listDirection: #topToBottom;
		 cornerStyle: #rounded.
	data := graph bars.
	numberOfBars := data size max: 10.
	self initializeSubmorphsWithData: data! !

!BarTableMorph methodsFor: 'building' stamp: 'Richo 5/13/2010 20:10'!
initializeSubmorphsWithData: data 
	"data should be a collection of points."
	self addMorphBack: self makeColumnNames;
		
		addMorphBack: (self makeListWithData: data);
		 addMorphBack: ((SimpleButtonMorph newWithLabel: '-') color: (Color
				r: 0.4
				g: 0.8
				b: 0.6) lighter;
			 borderWidth: 2;
			 width: 129;
			 target: self;
			 actionSelector: #removeBar);
		 addMorphBack: (Morph new color: Color transparent;
			 extent: 10 @ 3);
		 addMorphBack: ((SimpleButtonMorph newWithLabel: '+') color: (Color
				r: 0.4
				g: 0.8
				b: 0.6) lighter;
			 borderWidth: 2;
			 width: 129;
			 target: self;
			 actionSelector: #addBar);
		 addMorphBack: (Morph new color: Color transparent;
			 extent: 10 @ 3);
		 addMorphBack: ((SimpleButtonMorph newWithLabel: 'Apply') width: 129;
			 target: self;
			 actionSelector: #apply)! !

!BarTableMorph methodsFor: 'accessing' stamp: 'Richo 4/26/2010 23:08'!
list
	^list! !

!BarTableMorph methodsFor: 'building' stamp: 'Richo 6/17/2010 23:02'!
makeColumnNames
| columnName |
	columnName := [:contents | self newTextMorph contents: contents; wrapFlag: true; autoFit: true; backgroundColor: Color transparent; borderWidth: 0; centered].
	^ AlignmentMorph newRow color: Color transparent;
		addMorphBack: self makeSpacer;
		addMorphBack: (columnName value: 'Label');
		addMorphBack: self makeSpacer;
		addMorphBack: (columnName value: 'Value');
		addMorphBack: self makeSpacer;
		addMorphBack: (Morph new color: Color transparent; extent:  20@1)! !

!BarTableMorph methodsFor: 'building' stamp: 'Richo 6/17/2010 23:00'!
makeListWithData: data
	list := AlignmentMorph newColumn color: Color transparent.
	1 to: numberOfBars do: [:i| 
		list addMorphBack: (self makeRow: (data at: i ifAbsent: [nil]))].
	scrollPane := ScrollPane new extent: (self textMorphWidth * 2 ) + 40@281; borderWidth: 0.
	scrollPane scroller addMorph: list.
	data isEmpty ifFalse: [scrollPane vSetScrollDelta].
	^scrollPane! !

!BarTableMorph methodsFor: 'building' stamp: 'Richo 5/16/2010 13:16'!
makeRow: anAssoc 
	^ AlignmentMorph newRow color: Color transparent;
		 addMorphBack: (self newTextMorph
			contents: (anAssoc isNil
					ifTrue: ['']
					ifFalse: [anAssoc key asString]);
			 wrapFlag: true);
		 addMorphBack: (self newTextMorph
			contents: (anAssoc isNil
					ifTrue: ['']
					ifFalse: [anAssoc value isInteger ifFalse: [anAssoc value printShowingDecimalPlaces: 2] ifTrue: [anAssoc value asString]]);
			 wrapFlag: true) ! !

!BarTableMorph methodsFor: 'building' stamp: 'Richo 4/26/2010 23:08'!
makeSpacer
	^ AlignmentMorph newVariableTransparentSpacer! !

!BarTableMorph methodsFor: 'initialize' stamp: 'Richo 4/27/2010 12:41'!
newPlayerInstance
^ BarTablePlayer newUserInstance! !

!BarTableMorph methodsFor: 'building' stamp: 'Richo 5/16/2010 16:22'!
newTextMorph
	^ GSoCTextMorph new autoFit: false;
		 extent: self textMorphWidth @ 23;
		 borderWidth: 1;
		 backgroundColor: Color white;
		 yourself! !

!BarTableMorph methodsFor: 'actions' stamp: 'Richo 4/26/2010 23:17'!
removeBar
	| |
	numberOfBars <= 1 ifTrue: [^self].
	numberOfBars := numberOfBars - 1.
	list removeMorph: list submorphs last.
	self updateScrollPane! !

!BarTableMorph methodsFor: 'private' stamp: 'Richo 4/26/2010 23:08'!
setGraph: aDataGraphMorph
graph := aDataGraphMorph! !

!BarTableMorph methodsFor: 'private' stamp: 'Richo 5/13/2010 20:09'!
textMorphWidth
	^ 100! !

!BarTableMorph methodsFor: 'actions' stamp: 'Richo 4/26/2010 23:17'!
updateScrollPane
	scrollPane "hHideScrollBar;" vScrollBarValue: 1; vSetScrollDelta! !

!BarTableMorph methodsFor: 'accessing' stamp: 'Richo 5/16/2010 13:46'!
valueFromString: aString
 ^[Compiler evaluate: aString notifying: SyntaxErrorOmission new logged: false] on: Error do: [nil]! !

AlignmentMorph subclass: #PointTableMorph
	instanceVariableNames: 'numberOfPoints graph list scrollPane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!PointTableMorph commentStamp: 'Richo 4/17/2010 23:33' prior: 0!
This object is a table of points whose purpose is let the user fill the DataGraphMorph with points. !

!PointTableMorph class methodsFor: 'viewer categories' stamp: 'Richo 4/27/2010 12:44'!
additionsToViewerCategories
^ #(
(#plot (
#(command apply '' )
#(command addPoint '' )
#(command removePoint '' )
#(slot pointCount '' Number readOnly Player getPointCount Player unused )
#(slot pointIndex '' Number readWrite Player getPointIndex Player setPointIndex:  )
#(slot pointX '' Number readWrite Player getPointX Player setPointX:  )
#(slot pointY '' Number readWrite Player getPointY Player setPointY:  )
#(slot pointsFilled '' Number readOnly Player getPointsFilled Player unused )
))

)! !

!PointTableMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/11/2010 21:09'!
includeInNewMorphMenu
	"Return true for all classes that can be instantiated from the menu"
	^ false! !

!PointTableMorph class methodsFor: 'instance creation' stamp: 'Richo 4/25/2010 00:13'!
on: aDataGraphMorph
^(self basicNew setGraph: aDataGraphMorph) initialize ! !

!PointTableMorph methodsFor: 'actions' stamp: 'Richo 4/26/2010 22:50'!
addPoint
	| |
	numberOfPoints := numberOfPoints + 1.
"	self resetData."
	list addMorph: (self makeRow: nil)   asElementNumber: numberOfPoints + 1.
	self updateScrollPane.! !

!PointTableMorph methodsFor: 'actions' stamp: 'Richo 5/13/2010 20:42'!
apply
| newPoints oldPoint newPoint |
newPoints := self points.
1 to: newPoints size do: [:i|
	oldPoint := graph pointAt: i ifAbsent: [nil].
	newPoint := newPoints at: i.
	oldPoint isNil
	ifTrue: [graph addPoint: newPoint]
	ifFalse: [graph pointAt: i put: newPoint]
	].
newPoints size < graph pointCount
	ifTrue: [graph removePointsFrom: newPoints size + 1 to: graph pointCount].! !

!PointTableMorph methodsFor: 'initialize' stamp: 'Richo 4/15/2010 15:44'!
initialize
	| data |
	super initialize.
	self borderWidth: 0;
		
		color: ((Color
				r: 0.817
				g: 0.864
				b: 0.797)
				mixed: 0.75
				with: Color blue muchDarker);
		 hResizing: #shrinkWrap;
		 vResizing: #shrinkWrap;
		 listDirection: #topToBottom;
		 cornerStyle: #rounded.
	data := graph points.
	numberOfPoints := data size max: 10.
	self initializeSubmorphsWithData: data! !

!PointTableMorph methodsFor: 'building' stamp: 'Richo 5/13/2010 19:20'!
initializeSubmorphsWithData: data 
	"data should be a collection of points."
	self addMorphBack: self makeColumnNames;
		
		addMorphBack: (self makeListWithData: data);
		 addMorphBack: ((SimpleButtonMorph newWithLabel: '-') color: (Color
				r: 0.4
				g: 0.8
				b: 0.6) lighter;
			 borderWidth: 2;
			 width: (self textMorphWidth * 2) + 29;
			 target: self;
			 actionSelector: #removePoint);
		 addMorphBack: (Morph new color: Color transparent;
			 extent: 10 @ 3);
		 addMorphBack: ((SimpleButtonMorph newWithLabel: '+') color: (Color
				r: 0.4
				g: 0.8
				b: 0.6) lighter;
			 borderWidth: 2;
			 width:  (self textMorphWidth * 2) + 29;
			 target: self;
			 actionSelector: #addPoint);
		 addMorphBack: (Morph new color: Color transparent;
			 extent: 10 @ 3);
		 addMorphBack: ((SimpleButtonMorph newWithLabel: 'Apply') width:  (self textMorphWidth * 2) + 29;
			 target: self;
			 actionSelector: #apply)! !

!PointTableMorph methodsFor: 'accessing' stamp: 'Richo 4/25/2010 12:54'!
list
	^list! !

!PointTableMorph methodsFor: 'building' stamp: 'Richo 6/17/2010 23:01'!
makeColumnNames
	| columnName |
	columnName := [:contents | self newTextMorph contents: contents; wrapFlag: true; autoFit: true; backgroundColor: Color transparent; borderWidth: 0; centered].
	^ AlignmentMorph newRow color: Color transparent;
		addMorphBack: self makeSpacer;
		addMorphBack: (columnName value: 'x');
		addMorphBack: self makeSpacer;
		addMorphBack: (columnName value: 'y');
		addMorphBack: self makeSpacer;
		addMorphBack: (Morph new color: Color transparent; extent: 20@1)! !

!PointTableMorph methodsFor: 'building' stamp: 'Richo 6/17/2010 23:01'!
makeListWithData: data
	list := AlignmentMorph newColumn color: Color transparent.
	1 to: numberOfPoints do: [:i| 
		list addMorphBack: (self makeRow: (data at: i ifAbsent: [nil]))].
	scrollPane := ScrollPane new extent:  (self textMorphWidth * 2) + 40 @281; borderWidth: 0.
	scrollPane scroller addMorph: list.
	data isEmpty ifFalse: [scrollPane vSetScrollDelta].
	^scrollPane! !

!PointTableMorph methodsFor: 'building' stamp: 'Richo 5/16/2010 13:15'!
makeRow: aPoint 
	^ AlignmentMorph newRow color: Color transparent;
		 addMorphBack: (self newTextMorph
			contents: (aPoint isNil
					ifTrue: ['']
					ifFalse: [aPoint x isInteger ifFalse: [aPoint x printShowingDecimalPlaces: 2] ifTrue: [aPoint x asString]]);
			 wrapFlag: true);
		 addMorphBack: (self newTextMorph
			contents: (aPoint isNil
					ifTrue: ['']
					ifFalse: [aPoint y isInteger ifFalse: [aPoint y printShowingDecimalPlaces: 2] ifTrue: [aPoint y asString]]);
			 wrapFlag: true) ! !

!PointTableMorph methodsFor: 'building' stamp: 'Richo 4/4/2010 22:02'!
makeSpacer
	^ AlignmentMorph newVariableTransparentSpacer! !

!PointTableMorph methodsFor: 'initialize' stamp: 'Richo 4/16/2010 23:02'!
newPlayerInstance
^ PointTablePlayer newUserInstance! !

!PointTableMorph methodsFor: 'building' stamp: 'Richo 5/16/2010 16:22'!
newTextMorph
	^GSoCTextMorph new autoFit: false;
		 extent: self textMorphWidth @ 23;
		 borderWidth: 1;
		 backgroundColor: Color white;
		yourself! !

!PointTableMorph methodsFor: 'accessing' stamp: 'Richo 5/16/2010 16:41'!
points
"Searches across the submorphs and returns an array of points with the data written by the user"
	| data x y firstSubmorph secondSubmorph |
	data := OrderedCollection new.
	list
		submorphsDo: [:morph | morph isAlignmentMorph
				ifTrue: [
					x := y := nil.
					firstSubmorph := morph submorphs first.
					secondSubmorph := morph submorphs second.
					firstSubmorph isTextMorph
						ifTrue: [firstSubmorph contents asString = ''
								ifFalse: [x := self valueFromString: firstSubmorph contents asString]].
					secondSubmorph isTextMorph
						ifTrue: [secondSubmorph contents asString = ''
								ifFalse: [y := self valueFromString: secondSubmorph contents asString]].
					(x isNumber
							and: [y isNumber])
						ifTrue: [data add: x @ y]]].
	^ data! !

!PointTableMorph methodsFor: 'actions' stamp: 'Richo 4/26/2010 22:49'!
removePoint
	| |
	numberOfPoints <= 1 ifTrue: [^self].
	numberOfPoints := numberOfPoints - 1.
	list removeMorph: list submorphs last.
	self updateScrollPane! !

!PointTableMorph methodsFor: 'private' stamp: 'Richo 4/13/2010 12:22'!
setGraph: aDataGraphMorph
graph := aDataGraphMorph! !

!PointTableMorph methodsFor: 'private' stamp: 'Richo 5/13/2010 19:18'!
textMorphWidth
^75! !

!PointTableMorph methodsFor: 'actions' stamp: 'Richo 4/26/2010 23:17'!
updateScrollPane
	scrollPane vScrollBarValue: 1; vSetScrollDelta! !

!PointTableMorph methodsFor: 'accessing' stamp: 'Richo 5/16/2010 13:43'!
valueFromString: aString
 ^[Compiler evaluate: aString notifying: SyntaxErrorOmission new logged: false] on: Error do: [nil]! !

ScrollPane subclass: #GSoCScrollPane
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Utils'!

!GSoCScrollPane commentStamp: '<historical>' prior: 0!
A ScrollPane that defers its halo to interior!

!GSoCScrollPane methodsFor: 'as yet unclassified' stamp: 'Richo 6/17/2010 23:52'!
defersHaloToInteriorMorph: aMorph
^true! !

SketchMorph subclass: #PointMorph
	instanceVariableNames: 'graph cartesianPosition point'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!PointMorph commentStamp: 'Richo 4/17/2010 21:06' prior: 0!
This object is used by DataGraphMorph to draw points. It needs to know the graph in which it is plotted to be able to update its position whenever necessary.!

!PointMorph class methodsFor: 'viewer categories' stamp: 'Richo 4/26/2010 22:57'!
additionsToViewerCategories
^ #(
(#plot (
#(command remove '' )
#(slot pointX '' Number readWrite Player getPointX Player setPointX:  )
#(slot pointY '' Number readWrite Player getPointY Player setPointY:  )
#(slot pointColor '' Color readWrite Player getPointColor Player setPointColor:  )
))

)! !

!PointMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/21/2010 22:50'!
defaultNameStemForInstances
	^ 'Point' ! !

!PointMorph class methodsFor: 'instance creation' stamp: 'Richo 4/20/2010 23:49'!
graph: aDataGraphMorph point: aPoint 
^(self basicNew setGraph: aDataGraphMorph point: aPoint) initialize! !

!PointMorph methodsFor: 'accessing' stamp: 'Richo 4/21/2010 22:58'!
color
	^ color! !

!PointMorph methodsFor: 'accessing' stamp: 'Richo 4/21/2010 22:56'!
color: aColor
	"Set the receiver's color.  Directly set the color if appropriate, else go by way of fillStyle"

	(aColor isColor or: [aColor isKindOf: InfiniteForm]) ifFalse:[^ self fillStyle: aColor].
	color = aColor ifFalse:
		[self removeProperty: #fillStyle.
		color _ aColor.
		self changed].
	self initializeForm! !

!PointMorph methodsFor: 'initialize-release' stamp: 'Richo 4/27/2010 14:46'!
dismissViaHalo
	self remove.
	super dismissViaHalo! !

!PointMorph methodsFor: 'accessing' stamp: 'Richo 4/27/2010 11:56'!
fillStyle: aFillStyle
	"Set the current fillStyle of the receiver."
	self setProperty: #fillStyle toValue: aFillStyle.
	"Workaround for Morphs not yet converted"
	self color: aFillStyle asColor! !

!PointMorph methodsFor: 'initialize-release' stamp: 'Richo 4/25/2010 19:52'!
initialize
	super initialize.
	self color: Color red; initializeForm; update! !

!PointMorph methodsFor: 'initialize-release' stamp: 'Richo 4/27/2010 15:30'!
initializeForm
| tempForm |
tempForm := (Form extent: 6@6 depth: 32).
tempForm getCanvas fillOval: tempForm boundingBox color: self color.

self form: tempForm! !

!PointMorph methodsFor: 'initialize-release' stamp: 'Richo 4/16/2010 23:21'!
newPlayerInstance
^ PointPlayer newUserInstance! !

!PointMorph methodsFor: 'accessing' stamp: 'Richo 4/20/2010 23:51'!
point
^point! !

!PointMorph methodsFor: 'accessing' stamp: 'Richo 4/27/2010 15:18'!
point: aPoint
	self privateCenter: (graph cartesianPointToPixel: aPoint).
	point := aPoint! !

!PointMorph methodsFor: 'accessing' stamp: 'Richo 4/27/2010 15:51'!
position: aPoint
owner isNil ifFalse: [point := (graph relativePixelToCartesianPoint: aPoint + (self extent // 2) - graph topLeft)].
^super position: aPoint! !

!PointMorph methodsFor: 'private' stamp: 'Richo 4/27/2010 15:18'!
privateCenter: aPoint
self privatePosition: (aPoint - (self extent // 2))! !

!PointMorph methodsFor: 'private' stamp: 'Richo 4/27/2010 15:18'!
privatePosition: aPoint
^super position: aPoint! !

!PointMorph methodsFor: 'initialize-release' stamp: 'Richo 4/26/2010 23:01'!
remove
	graph removePoint: point! !

!PointMorph methodsFor: 'private' stamp: 'Richo 4/20/2010 23:49'!
setGraph: aDataGraphMorph point: aPoint 
graph := aDataGraphMorph.
point := aPoint.! !

!PointMorph methodsFor: 'updating' stamp: 'Richo 4/23/2010 16:26'!
update
self penUpWhile: [self point: point].
! !

!Morph class methodsFor: '*GSoC' stamp: 'Richo 6/17/2010 13:27'!
additionsToViewerCategoryGSoC
	"Answer viewer additions for the 'as object' category.  Just priming the pump at this point."

	^#(
		#'gsoc' 
		(
			(command say: String '')
			(command think: String '')
			(slot bubble '' Player readOnly Player getBubble Player unused)		
		))! !

!Morph methodsFor: '*GSoC' stamp: 'Richo 6/17/2010 00:01'!
bubble
	^self valueOfProperty: #bubble ifAbsent: [nil].! !

!Morph methodsFor: '*GSoC' stamp: 'Richo 6/16/2010 21:37'!
say: aString
self showMessage: aString inBubbleType: #speech! !

!Morph methodsFor: '*GSoC' stamp: 'Richo 6/17/2010 00:01'!
showMessage: aString inBubbleType: typeSymbol
| currentBubble |
currentBubble := self bubble.
currentBubble notNil ifTrue: [
	(currentBubble string = aString and: [currentBubble type = typeSymbol and: [currentBubble isInWorld]]) ifTrue: [^self].
	currentBubble delete].
aString isEmpty ifTrue: [^self removeProperty: #bubble].
self setProperty: #bubble toValue: (SpeechBubbleMorph string: aString type: typeSymbol for: self) openInWorld.! !

!Morph methodsFor: '*GSoC' stamp: 'Richo 6/16/2010 21:37'!
think: aString
self showMessage: aString inBubbleType: #thought! !

Morph subclass: #ScrollableSpreadsheetMorph
	instanceVariableNames: 'spreadsheet scrollPane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-IO'!

!ScrollableSpreadsheetMorph class methodsFor: 'viewer categories' stamp: 'Richo 6/18/2010 17:10'!
additionsToViewerCategories
^ (Smalltalk at: #SpreadsheetMorph ifAbsent: [^#()]) additionsToViewerCategories ! !

!ScrollableSpreadsheetMorph class methodsFor: 'parts bin' stamp: 'Richo 6/17/2010 22:37'!
descriptionForPartsBin
	"Answer a description for use in parts bins"

	^ self partName: 	'Spreadsheet' translatedNoop
		categories:		{'GSoC' translatedNoop}
		documentation:	'' translatedNoop! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:17'!
cellNumericValueAt: aPoint 
	^spreadsheet cellNumericValueAt: aPoint! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:19'!
cellNumericValueAt: aPoint put: aByteString 
	spreadsheet cellNumericValueAt: aPoint put: aByteString ! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:17'!
cellValueAt: aPoint 
	^spreadsheet cellValueAt: aPoint! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:18'!
cellValueAt: aPoint put: aByteString 
	spreadsheet cellValueAt: aPoint put: aByteString ! !

!ScrollableSpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 21:44'!
extent: aPoint
| newExtent |
newExtent := (aPoint x max: 126) @ (aPoint y max: 102).
scrollPane extent: newExtent.
super extent: newExtent
! !

!ScrollableSpreadsheetMorph methodsFor: 'halo' stamp: 'Richo 6/18/2010 10:19'!
handlerForBlueButtonDown: anEvent
"It defers the halo to interior only if the click is inside the bounds of the spreadsheet"
^(spreadsheet boundsInWorld containsPoint: anEvent cursorPoint)
	ifTrue: [nil] ifFalse: [self]
! !

!ScrollableSpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/18/2010 00:04'!
initialize
super initialize.
scrollPane := GSoCScrollPane new extent:  620@300; borderWidth: 0; color: (Color r: 0.419 g: 0.419 b: 0.419).
scrollPane scroller addMorph: (spreadsheet := (SpreadsheetMorph rows: 10 columns: 5) position: 2@5).
scrollPane alwaysShowScrollBars: true.
self extent: scrollPane extent; color: Color transparent.
self addMorph: scrollPane.! !

!ScrollableSpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 21:18'!
newPlayerInstance
	^spreadsheet newPlayerInstance! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 21:19'!
numberOfColumns
^spreadsheet numberOfColumns! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 21:20'!
numberOfColumns: aNumber
	spreadsheet numberOfColumns: aNumber.
	self updateScrollPane! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 21:17'!
numberOfRows
^spreadsheet numberOfRows! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 21:20'!
numberOfRows: aNumber
	spreadsheet numberOfRows: aNumber.
	self updateScrollPane! !

!ScrollableSpreadsheetMorph methodsFor: 'i/o' stamp: 'Richo 6/17/2010 22:46'!
openCSVFile: filePath
	spreadsheet openCSVFile: filePath! !

!ScrollableSpreadsheetMorph methodsFor: 'i/o' stamp: 'Richo 6/18/2010 11:57'!
openCSVFile: filePath separator: aCharacter
	spreadsheet openCSVFile: filePath separator: aCharacter! !

!ScrollableSpreadsheetMorph methodsFor: 'i/o' stamp: 'Richo 6/18/2010 14:28'!
saveCSVFile: filePath separator: aCharacter
	spreadsheet saveCSVFile: filePath separator: aCharacter! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 21:49'!
updateScrollPane
	scrollPane vSetScrollDelta; hSetScrollDelta! !

Morph subclass: #SpeechBubbleMorph
	instanceVariableNames: 'string type text balloon tail target lastHash stepTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-SpeechBubble'!

SpeechBubbleMorph class
	instanceVariableNames: 'speakingForm topLeftCornerForm thinkingForm'!

SpeechBubbleMorph class
	instanceVariableNames: 'speakingForm topLeftCornerForm thinkingForm'!

!SpeechBubbleMorph class methodsFor: 'viewer categories' stamp: 'Richo 6/16/2010 21:44'!
additionsToViewerCategoryBubble
	"Answer viewer additions for the 'bubble' category"

	^#(
		bubble 
		(
			(command attachTo: '' Player)
			(command stopAttaching '')
			(slot attachment '' Player readOnly Player getAttachment Player unused)		
		)
	)
! !

!SpeechBubbleMorph class methodsFor: 'forms' stamp: 'Richo 6/14/2010 11:08'!
bottomLeftCornerForm
^self topLeftCornerForm flipBy: #vertical centerAt: self topLeftCornerForm boundingBox topCenter! !

!SpeechBubbleMorph class methodsFor: 'forms' stamp: 'Richo 6/14/2010 11:08'!
bottomRightCornerForm
^(self topLeftCornerForm flipBy: #horizontal centerAt: self topLeftCornerForm boundingBox leftCenter) flipBy: #vertical centerAt: self topLeftCornerForm boundingBox topCenter! !

!SpeechBubbleMorph class methodsFor: 'instance creation' stamp: 'Richo 6/14/2010 15:11'!
new
	^self string: 'Hello world!!'! !

!SpeechBubbleMorph class methodsFor: 'forms' stamp: 'Richo 6/16/2010 21:10'!
speakingForm
"
speakingForm := nil
"
^speakingForm ifNil: [speakingForm := (Form
	extent: 56@51
	depth: 16
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147418113 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 65537 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 65537 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)]! !

!SpeechBubbleMorph class methodsFor: 'instance creation' stamp: 'Richo 6/16/2010 21:33'!
speechPrototype
	^self string: 'Hello world!!' type: #speech! !

!SpeechBubbleMorph class methodsFor: 'instance creation' stamp: 'Richo 6/14/2010 15:12'!
string: aString 
^self string: aString type: #speech! !

!SpeechBubbleMorph class methodsFor: 'instance creation' stamp: 'Richo 6/14/2010 15:12'!
string: aString type: aSymbol
"self string: 'Hello world!!' type: #speech"
^(self basicNew setString: aString type: aSymbol) initialize! !

!SpeechBubbleMorph class methodsFor: 'instance creation' stamp: 'Richo 6/14/2010 15:12'!
string: aString type: aSymbol for: aMorph
"self string: 'Hello world!!' type: #speech for: Morph new openInHand"
^(self string: aString type: aSymbol) target: aMorph! !

!SpeechBubbleMorph class methodsFor: 'parts bin' stamp: 'Richo 6/16/2010 21:32'!
supplementaryPartsDescriptions
	"Answer a list of DescriptionForPartsBin objects that characterize objects that this class wishes to contribute to Stationery bins *other* than by the standard default #newStandAlone protocol"

	^ {
	DescriptionForPartsBin
		formalName: 'Speech bubble' translatedNoop
		categoryList: #(#GSoC)
		documentation: '' translatedNoop
		globalReceiverSymbol: #SpeechBubbleMorph
		nativitySelector: #speechPrototype.

	DescriptionForPartsBin
		formalName: 'Thought bubble' translatedNoop
		categoryList: #(#GSoC)
		documentation: '' translatedNoop
		globalReceiverSymbol: #SpeechBubbleMorph
		nativitySelector: #thoughtPrototype.
}
! !

!SpeechBubbleMorph class methodsFor: 'forms' stamp: 'Richo 6/16/2010 21:07'!
thinkingForm
"
thinkingForm := nil
thinkingForm
"
^thinkingForm ifNil: [thinkingForm := (Form
	extent: 56@49
	depth: 16
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 65537 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 65537 65537 65537 65537 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 65537 65537 2147450879 2147450879 2147450879 65537 65537 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 65537 65537 65537 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 1 65537 65537 65537 65537 65537 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 65537 65537 98303 2147450879 2147418113 65537 65537 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147418113 65537 65536 1 65537 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 65536 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65536 0 65537 65537 65537 2147450879 2147450879 2147450879 65537 65537 65537 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65536 0 0 1 65537 65537 65537 65537 65537 65537 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 1 65537 65537 65537 65536 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147418113 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 65537 65536 0 1 65537 98303 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 65537 65537 65537 65536 0 65537 65537 98303 2147450879 2147418113 65537 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 65537 65536 0 0 65537 65537 65537 65537 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147418113 65537 0 0 0 65537 65537 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 2147450879 2147450879 2147450879 2147450879 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 65537 98303 2147450879 2147450879 2147418113 65537 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 2147450879 2147450879 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 65537 65537 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 65537 65537 65536 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)]! !

!SpeechBubbleMorph class methodsFor: 'instance creation' stamp: 'Richo 6/16/2010 21:33'!
thoughtPrototype
	^self string: 'Hello world!!' type: #thought! !

!SpeechBubbleMorph class methodsFor: 'forms' stamp: 'Richo 6/16/2010 20:50'!
topLeftCornerForm
"
topLeftCornerForm := nil
(SketchMorph withForm: topLeftCornerForm) openInHand
"
^topLeftCornerForm ifNil: [topLeftCornerForm := (Form
	extent: 25@25
	depth: 16
	fromArray: #( 0 0 0 0 0 0 0 1 65537 65537 65537 65537 65536 0 0 0 0 0 1 65537 65537 65537 65537 65537 65537 65536 0 0 0 0 1 65537 65537 65537 2147450879 2147450879 2147450879 2147450879 2147418112 0 0 0 1 65537 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 0 0 0 65537 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 0 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 0 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 0 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 0 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 0 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 0 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 0 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 1 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 1 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 65537 98303 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112 65537 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147450879 2147418112)
	offset: 0@0)]! !

!SpeechBubbleMorph class methodsFor: 'forms' stamp: 'Richo 6/14/2010 11:15'!
topRightCornerForm
^self topLeftCornerForm flipBy: #horizontal centerAt:  self topLeftCornerForm boundingBox leftCenter! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/16/2010 21:23'!
balloon
	^balloon ifNil: [
		| balloonForm |
		balloonForm := Form extent: self extent - (0 @ self tailHeight) depth: 16.
		self drawBalloonOn: balloonForm getCanvas in: balloonForm boundingBox.
		balloonForm floodFill: self color at: balloonForm center.
		balloon := (SketchMorph withForm: balloonForm).
	]! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/16/2010 21:23'!
color: aColor
	super color: aColor.
	self refresh! !

!SpeechBubbleMorph methodsFor: 'testing' stamp: 'Richo 6/14/2010 16:41'!
containsPoint: aPoint
^ (self bounds containsPoint: aPoint) and:
	  [(self imageForm isTransparentAt: aPoint - bounds origin) not]
! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/16/2010 21:02'!
defaultColor
	^Color white! !

!SpeechBubbleMorph methodsFor: 'drawing' stamp: 'Richo 6/14/2010 15:20'!
drawBalloonOn: aCanvas in: sourceRect
| cornerBounds rect1 rect2 |
cornerBounds := self class topLeftCornerForm boundingBox.
aCanvas translucentImage: self class topLeftCornerForm at: sourceRect topLeft;
		translucentImage: self class topRightCornerForm at: sourceRect topRight - (cornerBounds width @ 0);
		translucentImage: self class bottomLeftCornerForm at: sourceRect bottomLeft - (0 @ (cornerBounds height));
		translucentImage: self class bottomRightCornerForm at: sourceRect bottomRight - cornerBounds extent.

rect1 := sourceRect topLeft + (cornerBounds width @ 1) corner: sourceRect bottomRight - (cornerBounds width @ 1).
rect2 := sourceRect topLeft + (1 @ cornerBounds height) corner: sourceRect bottomRight - (1 @ cornerBounds height).
aCanvas fillRectangle: rect1 color: Color white; fillRectangle: rect2 color: Color white.
aCanvas line: rect1 topLeft to: rect1 topRight width: 2 color: Color black;
		line: rect1 bottomLeft to: rect1 bottomRight width: 2 color: Color black;
		line: rect2 topLeft to: rect2 bottomLeft width: 2 color: Color black;
		line: rect2 topRight to: rect2 bottomRight width: 2 color: Color black.
! !

!SpeechBubbleMorph methodsFor: 'drawing' stamp: 'Richo 6/16/2010 20:56'!
drawOn: aCanvas! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 12:43'!
extent: aPoint
| width height |
width := aPoint x max: self minimumAcceptedWidth.
height := aPoint y max: self minimumAcceptedHeight.
super extent: width @ height.
self refresh.
target notNil ifTrue: [self positionMyselfAccordingToTarget]! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/16/2010 20:59'!
fillStyle: aFillStyle
	super fillStyle: aFillStyle.
	self refresh! !

!SpeechBubbleMorph methodsFor: 'stepping' stamp: 'Richo 6/14/2010 16:31'!
incrementStepTime
	stepTime := (stepTime + 1) min: self maximumStepTime! !

!SpeechBubbleMorph methodsFor: 'initialization' stamp: 'Richo 6/16/2010 21:01'!
initialize
| |
super initialize.
stepTime := self minimumStepTime.
self extent: self textExtent + (20 @ self tailHeight + 20); color: Color white.
self positionBalloon; positionTail; positionText.
self addMorph: self balloon; addMorph: self tail; addMorph: self text! !

!SpeechBubbleMorph methodsFor: 'event handling' stamp: 'Richo 6/17/2010 13:25'!
keyStroke: anEvent morph: aMorph 

	aMorph == self text
		ifFalse: ["I'm not sure this check is needed..."^ self].

	string := self text contents.

	"Update text width if necessary. Make sure we keep the selection at the end of the text so that the user can keep modifying"
	self text width > 300
		ifTrue: [self text contents: string wrappedTo: 300.
			self text editor selectFrom: string size + 1 to: string size].

	"Update my extent"
	self extent: self textExtent + (20 @ self tailHeight + 20).
! !

!SpeechBubbleMorph methodsFor: 'stepping' stamp: 'Richo 6/17/2010 00:11'!
maximumStepTime
	^500! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/16/2010 21:01'!
minimumAcceptedHeight
^100 max: self text height + 20 + self tailHeight! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/16/2010 21:01'!
minimumAcceptedWidth
^ 175 max: self text width + 20! !

!SpeechBubbleMorph methodsFor: 'stepping' stamp: 'Richo 6/14/2010 16:32'!
minimumStepTime
	^20! !

!SpeechBubbleMorph methodsFor: 'WiW support' stamp: 'Richo 6/14/2010 16:53'!
morphicLayerNumber

	"helpful for insuring some morphs always appear in front of or behind others.
	smaller numbers are in front"

	^5		"Balloons are very front-like things"! !

!SpeechBubbleMorph methodsFor: 'initialization' stamp: 'Richo 6/16/2010 20:34'!
newPlayerInstance
	^SpeechBubblePlayer newUserInstance
! !

!SpeechBubbleMorph methodsFor: 'initialization' stamp: 'Richo 6/14/2010 15:41'!
positionBalloon
self balloon position: self position! !

!SpeechBubbleMorph methodsFor: 'stepping' stamp: 'Richo 6/14/2010 16:32'!
positionMyselfAccordingToTarget
	self center: target topRendererOrSelf center - (0 @ ((target topRendererOrSelf height + self height) / 2)).! !

!SpeechBubbleMorph methodsFor: 'initialization' stamp: 'Richo 6/14/2010 15:40'!
positionTail
self tail position: self bottomCenter - (0 @ self tailHeight + 2)! !

!SpeechBubbleMorph methodsFor: 'initialization' stamp: 'Richo 6/14/2010 15:42'!
positionText
self text center: self center - (0 @ self tailHeight / 2).! !

!SpeechBubbleMorph methodsFor: 'refreshing' stamp: 'Richo 6/16/2010 20:58'!
refresh
self refreshBalloon; refreshTail; refreshText.! !

!SpeechBubbleMorph methodsFor: 'refreshing' stamp: 'Richo 6/14/2010 15:49'!
refreshBalloon
	balloon ifNotNil: [balloon delete].
	balloon := nil.
	self positionBalloon.
	self addMorph: balloon! !

!SpeechBubbleMorph methodsFor: 'refreshing' stamp: 'Richo 6/14/2010 15:50'!
refreshTail
	tail ifNotNil: [tail delete].
	tail := nil.
	self positionTail.
	self addMorph: tail! !

!SpeechBubbleMorph methodsFor: 'refreshing' stamp: 'Richo 6/17/2010 13:23'!
refreshText
	self positionText.
	self addMorph: self text! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/14/2010 15:19'!
selectedTailForm
	^type caseOf: {
		[#speech] -> [self class speakingForm].
		[#thought] -> [self class thinkingForm].
		} otherwise: [self error: 'Wrong type']! !

!SpeechBubbleMorph methodsFor: 'private' stamp: 'Richo 6/14/2010 15:13'!
setString: msgString type: typeSymbol
string := msgString.
type := typeSymbol.! !

!SpeechBubbleMorph methodsFor: 'stepping' stamp: 'Richo 6/16/2010 21:49'!
step
(target isNil or: [lastHash = (lastHash := target boundsSignatureHash)])
		ifTrue: [self incrementStepTime]
		ifFalse: [
			target isInWorld ifFalse: [self delete].
			self positionMyselfAccordingToTarget.
			stepTime := self minimumStepTime 
		].
! !

!SpeechBubbleMorph methodsFor: 'stepping' stamp: 'Richo 6/14/2010 16:34'!
stepTime
^stepTime ! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/14/2010 16:43'!
string
^string! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/16/2010 21:24'!
tail
^tail ifNil: [
	| tailForm |
	tailForm := self selectedTailForm deepCopy.
	
	"This will paint both forms correctly"
	tailForm floodFill: self color at: tailForm center + (6@-15).

	"In the #thought case, we also need to paint the little bubbles"
	type = #thought ifTrue: [
		tailForm floodFill: self color at: tailForm center + (-7@7);
				floodFill: self color at: tailForm center + (-22@20)
	].

	tail := SketchMorph withForm: tailForm]! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/14/2010 15:36'!
tailHeight
^self tail height! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/16/2010 23:58'!
target
	^target! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/16/2010 21:47'!
target: aMorph
	target := aMorph.
	target notNil ifTrue: [self positionMyselfAccordingToTarget ]! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 13:12'!
text
^text ifNil: [
	text := (UserText new contents: string) centered.
	text width > 300 ifTrue: [text contents: string wrappedTo: 300].
	text on: #keyStroke send: #keyStroke:morph: to: self
]! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/14/2010 15:32'!
textExtent
	^self text extent! !

!SpeechBubbleMorph methodsFor: 'accessing' stamp: 'Richo 6/14/2010 16:43'!
type
^type! !

Morph subclass: #SpreadsheetMorph
	instanceVariableNames: 'numberOfRows numberOfColumns'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-IO'!

!SpreadsheetMorph class methodsFor: 'viewer categories' stamp: 'Richo 6/18/2010 14:33'!
additionsToViewerCategories
^ #(
(#spreadsheet (
#(slot totalRows '' Number readWrite Player getTotalRows Player setTotalRows:  )
#(slot totalColumns '' Number readWrite Player getTotalColumns Player setTotalColumns:  )
#(slot rowIndex '' Number readWrite Player getRowIndex Player setRowIndex:  )
#(slot columnIndex '' Number readWrite Player getColumnIndex Player setColumnIndex:  )
#(slot currentCharacters '' String readWrite Player getCurrentCharacters Player setCurrentCharacters:  )
#(slot currentNumericValue '' Number readWrite Player getCurrentNumericValue Player setCurrentNumericValue:  )
))

(#'file i/o' (
#(slot csvSeparator '' String readWrite Player getCsvSeparator Player setCsvSeparator:  )
#(slot csvFileName '' String readWrite Player getCsvFileName Player setCsvFileName:  )
#(command loadFile '')
#(command saveFile '')
))

)! !

!SpreadsheetMorph class methodsFor: 'instance creation' stamp: 'Richo 6/17/2010 16:44'!
rows: numberOfRows columns: numberOfColumns
	^(self basicNew setRows: numberOfRows columns: numberOfColumns) initialize! !

!SpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 18:33'!
addColumn
	numberOfColumns := numberOfColumns + 1.
	self submorphsDo: [:row | row addCell]! !

!SpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 20:48'!
addColumns: aNumber
	aNumber timesRepeat: [self addColumn]! !

!SpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/18/2010 10:42'!
addRow
	| newRow |
	numberOfRows := numberOfRows + 1.
	newRow := self newRow.
	1 to: numberOfColumns do: [:i | 
		newRow column: i width: (self widthOfCell: i)	
	].
	self addMorphBack: newRow! !

!SpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 20:48'!
addRows: aNumber
	aNumber timesRepeat: [self addRow]! !

!SpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:14'!
cellNumericValueAt: aPoint
^(submorphs at: aPoint x) cellNumericValueAt: aPoint y! !

!SpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:14'!
cellNumericValueAt: aPoint put: aNumber
 (submorphs at: aPoint x) cellNumericValueAt: aPoint y put: aNumber! !

!SpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:11'!
cellValueAt: aPoint
^(submorphs at: aPoint x) cellValueAt: aPoint y! !

!SpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:11'!
cellValueAt: aPoint put: aString
 (submorphs at: aPoint x) cellValueAt: aPoint y put: aString! !

!SpreadsheetMorph methodsFor: 'cell extent change' stamp: 'Richo 6/17/2010 23:45'!
column: index width: aNumber
self submorphsDo: [:each| each column: index width: aNumber]! !

!SpreadsheetMorph methodsFor: 'halo' stamp: 'Richo 6/17/2010 23:48'!
defersHaloToInteriorMorph: aMorph
^true! !

!SpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 19:18'!
initialize
super initialize.
self layoutPolicy: TableLayout new; listDirection: #topToBottom.
self hResizing: #shrinkWrap;
	vResizing: #shrinkWrap.
self color: Color black.
self initializeSubmorphs! !

!SpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 19:21'!
initializeSubmorphs
numberOfRows timesRepeat: [self addMorph: self newRow]! !

!SpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 20:53'!
newPlayerInstance
	^SpreadsheetPlayer newUserInstance! !

!SpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 19:21'!
newRow
	^(SpreadsheetRowMorph numberOfCells: numberOfColumns)! !

!SpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 20:46'!
numberOfColumns
^numberOfColumns ! !

!SpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 20:52'!
numberOfColumns: aNumber
| diff |
aNumber <= 0 ifTrue: [^self error: 'Spreadsheet must contain at least one column'].
diff := aNumber - numberOfColumns.
diff > 0 ifTrue: [self addColumns: diff] ifFalse: [self removeColumns: diff negated]
! !

!SpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 20:46'!
numberOfRows
^numberOfRows ! !

!SpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 20:52'!
numberOfRows: aNumber
| diff |
aNumber <= 0 ifTrue: [^self error: 'Spreadsheet must contain at least one row'].
diff := aNumber - numberOfRows.
diff > 0 ifTrue: [self addRows: diff] ifFalse: [self removeRows: diff negated]
! !

!SpreadsheetMorph methodsFor: 'i/o' stamp: 'Richo 6/17/2010 21:53'!
openCSVFile: filePath
self openCSVFile: filePath separator: $,! !

!SpreadsheetMorph methodsFor: 'i/o' stamp: 'Richo 6/18/2010 17:50'!
openCSVFile: filePath separator: separator
| values lines rows columns lineValues file |
file := FileStream fileNamed: filePath.
[	values := OrderedCollection new.
	lines := file contents subStrings: {Character cr. Character lf}.
	rows := lines size.
	columns := 1.
	lines do: [:line| 
		lineValues := (line subStrings: (Array with: separator)) collect: [:each | each withBlanksCondensed].
		columns := (columns max: lineValues size).
		values add: lineValues.
	].
] ensure: [file close].

self numberOfRows: rows; numberOfColumns: columns.
1 to: rows do: [:r|
	1 to: columns do: [:c |
		self cellValueAt: r@c put: ((values at: r) at: c ifAbsent: '')
	]
]! !

!SpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 18:35'!
removeColumn
	numberOfColumns := numberOfColumns - 1.
	self submorphsDo: [:row | row removeCell]! !

!SpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 20:49'!
removeColumns: aNumber
	aNumber timesRepeat: [self removeColumn]! !

!SpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 18:38'!
removeRow
	numberOfRows := numberOfRows - 1.
	self removeMorph: self submorphs last! !

!SpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 20:48'!
removeRows: aNumber
	aNumber timesRepeat: [self removeRow]! !

!SpreadsheetMorph methodsFor: 'i/o' stamp: 'Richo 6/18/2010 14:36'!
saveCSVFile: filePath separator: separator
| file |
file := FileStream forceNewFileNamed: filePath.
[1 to: numberOfRows do: [:r|
	1 to: numberOfColumns do: [:c |
		file nextPutAll: (self cellValueAt: r@c).
		c = numberOfColumns ifFalse: [file nextPut: separator].
	].
	file cr.
]
] ensure: [file close]! !

!SpreadsheetMorph methodsFor: 'private' stamp: 'Richo 6/17/2010 16:45'!
setRows: nRows columns: nColumns
	numberOfRows := nRows.
	numberOfColumns := nColumns! !

!SpreadsheetMorph methodsFor: 'cell extent change' stamp: 'Richo 6/18/2010 10:42'!
widthOfCell: index
	^((submorphs first) submorphs at: index) width! !

Morph subclass: #SpreadsheetRowMorph
	instanceVariableNames: 'numberOfCells'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-IO'!

!SpreadsheetRowMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 6/17/2010 16:19'!
numberOfCells: aNumber
^(self basicNew setNumberOfCells: aNumber) initialize! !

!SpreadsheetRowMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/18/2010 10:47'!
addCell
	numberOfCells := numberOfCells + 1.
	self addMorphBack: ((self newTextMorphWithContents: '') height: self height)! !

!SpreadsheetRowMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:16'!
cellNumericValueAt: index
 ^[Compiler evaluate: (submorphs at: index) contents notifying: SyntaxErrorOmission new logged: false] on: Error do: [nil]! !

!SpreadsheetRowMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:16'!
cellNumericValueAt: index put: aNumber
(submorphs at: index) contentsWrapped: aNumber asString! !

!SpreadsheetRowMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:11'!
cellValueAt: index
^(submorphs at: index) contents! !

!SpreadsheetRowMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 22:14'!
cellValueAt: index put: aString
(submorphs at: index) contentsWrapped: aString! !

!SpreadsheetRowMorph methodsFor: 'cell extent change' stamp: 'Richo 6/17/2010 23:46'!
column: index width: aNumber
(submorphs at: index) width: aNumber! !

!SpreadsheetRowMorph methodsFor: 'halo' stamp: 'Richo 6/17/2010 23:48'!
defersHaloToInteriorMorph: aMorph
^true! !

!SpreadsheetRowMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 19:18'!
initialize
super initialize.
self layoutPolicy: TableLayout new; listDirection: #leftToRight.
self hResizing: #shrinkWrap;
	vResizing: #shrinkWrap.
self color: Color black.
self initializeSubmorphs! !

!SpreadsheetRowMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 18:38'!
initializeSubmorphs
	numberOfCells timesRepeat: [self addMorphBack:(self newTextMorphWithContents: '')]! !

!SpreadsheetRowMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 20:39'!
newTextMorphWithContents: aString
	^ GSoCTextMorph new autoFit: false;
		 extent: 100 @ 23;
		 borderWidth: 1;
		 backgroundColor: Color white;
		 contentsWrapped: aString! !

!SpreadsheetRowMorph methodsFor: 'cell extent change' stamp: 'Richo 6/17/2010 23:47'!
notifyExtentChange: aMorph
| index |
index := self submorphIndexOf: aMorph.
self submorphsDo: [:each | each height: aMorph height].
owner column: index width: aMorph width.! !

!SpreadsheetRowMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 18:36'!
removeCell
	numberOfCells := numberOfCells - 1.
	self removeMorph: self submorphs last! !

!SpreadsheetRowMorph methodsFor: 'private' stamp: 'Richo 6/17/2010 16:20'!
setNumberOfCells: aSmallInteger 
	numberOfCells := aSmallInteger! !

Player subclass: #BarPlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!BarPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/24/2010 22:11'!
isUniClass
	^ self ~~ self officialClass! !

!BarPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/24/2010 22:11'!
officialClass
	^ PointPlayer! !

!BarPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/24/2010 22:11'!
wantsChangeSetLogging
	^ (self == self officialClass
			or: [(self name beginsWith: self officialClass asString) not])
		or: [Preferences universalTiles]! !

!BarPlayer methodsFor: 'slots' stamp: 'Richo 4/24/2010 22:54'!
getBarColor
^ self costume renderedMorph color! !

!BarPlayer methodsFor: 'slots' stamp: 'Richo 4/24/2010 22:56'!
getBarLabel
^ self costume renderedMorph bar key! !

!BarPlayer methodsFor: 'slots' stamp: 'Richo 4/24/2010 22:56'!
getBarValue
^ self costume renderedMorph bar value! !

!BarPlayer methodsFor: 'commands' stamp: 'Richo 5/13/2010 20:19'!
remove
^ self costume renderedMorph remove! !

!BarPlayer methodsFor: 'slots' stamp: 'Richo 4/24/2010 22:55'!
setBarColor: aColor
^ self costume renderedMorph color: aColor! !

!BarPlayer methodsFor: 'slots' stamp: 'Richo 4/24/2010 22:56'!
setBarLabel: aString
| morph |
morph := self costume renderedMorph.
morph bar: aString -> morph bar value.! !

!BarPlayer methodsFor: 'slots' stamp: 'Richo 4/24/2010 22:55'!
setBarValue: aNumber
| morph |
morph := self costume renderedMorph.
morph bar: morph bar key -> aNumber.! !

Player subclass: #BarTablePlayer
	instanceVariableNames: 'barIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!BarTablePlayer class methodsFor: 'change set logging' stamp: 'Richo 4/27/2010 12:42'!
isUniClass
	^ self ~~ self officialClass! !

!BarTablePlayer class methodsFor: 'change set logging' stamp: 'Richo 4/27/2010 12:43'!
officialClass
	^ BarTablePlayer! !

!BarTablePlayer class methodsFor: 'change set logging' stamp: 'Richo 4/27/2010 12:43'!
wantsChangeSetLogging
	^ (self == self officialClass
			or: [(self name beginsWith: self officialClass asString) not])
		or: [Preferences universalTiles]! !

!BarTablePlayer methodsFor: 'commands' stamp: 'Richo 4/27/2010 12:44'!
addBar
^ self costume renderedMorph addBar! !

!BarTablePlayer methodsFor: 'commands' stamp: 'Richo 4/27/2010 12:43'!
apply
^ self costume renderedMorph apply! !

!BarTablePlayer methodsFor: 'slots' stamp: 'Richo 4/27/2010 12:45'!
getBarCount
^( self costume renderedMorph list submorphs select: [:each| each isAlignmentMorph and: [each submorphs first isTextMorph]]) size! !

!BarTablePlayer methodsFor: 'slots' stamp: 'Richo 4/27/2010 12:46'!
getBarIndex
barIndex > self getBarCount ifTrue: [barIndex := self getBarCount].
^ barIndex! !

!BarTablePlayer methodsFor: 'slots' stamp: 'Richo 4/27/2010 12:53'!
getBarLabel
| morph count |
morph := self costume renderedMorph.
count := 0.
morph list submorphs do: [:each|
	(each isAlignmentMorph and: [ each submorphs first isTextMorph])
	 ifTrue: [
		count := count + 1.
		count = barIndex ifTrue: [
			^each submorphs first contents asString
			]			
		]
	]! !

!BarTablePlayer methodsFor: 'slots' stamp: 'Richo 4/27/2010 12:46'!
getBarsFilled
^ self costume renderedMorph bars size! !

!BarTablePlayer methodsFor: 'slots' stamp: 'Richo 5/13/2010 19:07'!
getBarValue
| morph count |
morph := self costume renderedMorph.
count := 0.
morph list submorphs do: [:each|
	(each isAlignmentMorph and: [ each submorphs first isTextMorph])
	 ifTrue: [
		count := count + 1.
		count = barIndex ifTrue: [
			^[each submorphs second contents asString asNumber] on: Error do: [0]
			]			
		]
	]! !

!BarTablePlayer methodsFor: 'initialize-release' stamp: 'Richo 4/27/2010 12:45'!
initialize
super initialize.
barIndex := 1.! !

!BarTablePlayer methodsFor: 'commands' stamp: 'Richo 4/27/2010 12:44'!
removeBar
^ self costume renderedMorph removeBar! !

!BarTablePlayer methodsFor: 'slots' stamp: 'Richo 4/27/2010 12:46'!
setBarIndex: aNumber 
self getBarCount = 0 
	ifTrue: [barIndex := 0. ^self].
(aNumber > self getBarCount)
	ifTrue: [barIndex := 1.^self].
aNumber < 1
	ifTrue: [barIndex := self getBarCount.^self].
barIndex := aNumber.! !

!BarTablePlayer methodsFor: 'slots' stamp: 'Richo 4/27/2010 12:50'!
setBarLabel: aString
| morph count |
morph := self costume renderedMorph.
count := 0.
morph list submorphs do: [:each|
	(each isAlignmentMorph and: [ each submorphs first isTextMorph])
	 ifTrue: [
		count := count + 1.
		count = barIndex ifTrue: [
			each submorphs first contents: aString asString.
			^self
			]			
		]
	]! !

!BarTablePlayer methodsFor: 'slots' stamp: 'Richo 4/27/2010 12:52'!
setBarValue: aNumber
| morph count |
morph := self costume renderedMorph.
count := 0.
morph list submorphs do: [:each|
	(each isAlignmentMorph and: [ each submorphs first isTextMorph])
	 ifTrue: [
		count := count + 1.
		count = barIndex ifTrue: [
			each submorphs second contents: aNumber asString.
			^self
			]			
		]
	]! !

Player subclass: #CartesianGraphPlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!CartesianGraphPlayer commentStamp: 'Richo 4/17/2010 20:52' prior: 0!
This class is needed to add Etoys scripting capabitlities to DataGraphMorph.!

!CartesianGraphPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/15/2010 12:45'!
isUniClass
	^ self ~~ self officialClass! !

!CartesianGraphPlayer class methodsFor: 'change set logging' stamp: 'Richo 5/21/2010 08:23'!
officialClass
	^ CartesianGraphPlayer! !

!CartesianGraphPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/15/2010 12:45'!
wantsChangeSetLogging
	^ (self == self officialClass
			or: [(self name beginsWith: self officialClass asString) not])
		or: [Preferences universalTiles]! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 15:41'!
getAxisColor 
^ self costume renderedMorph axisColor! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 15:41'!
getBackgroundColor 
^ self costume renderedMorph backgroundColor! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getMajorGridColor
^ self costume renderedMorph majorGridColor! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getMinorGridColor
^ self costume renderedMorph minorGridColor! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getShowCursorPosition
^ self costume renderedMorph showCursorPosition! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getXLegendSpacing
^ self costume renderedMorph legendSpacing x! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getXMajorGrid
^ self costume renderedMorph majorGrid x! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getXMax
^ self costume renderedMorph cartesianBounds right! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getXMin
^ self costume renderedMorph cartesianBounds left! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getXMinorGrid
^ self costume renderedMorph minorGrid x! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getYLegendSpacing
^ self costume renderedMorph legendSpacing y! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getYMajorGrid
^ self costume renderedMorph majorGrid y! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getYMax
^ self costume renderedMorph cartesianBounds bottom! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getYMin
	^self costume renderedMorph cartesianBounds top! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
getYMinorGrid
^ self costume renderedMorph minorGrid y! !

!CartesianGraphPlayer methodsFor: 'commands' stamp: 'Richo 4/15/2010 12:45'!
moveDown: aNumber
^ self costume renderedMorph move: 0 @ (aNumber negated)! !

!CartesianGraphPlayer methodsFor: 'commands' stamp: 'Richo 4/15/2010 12:45'!
moveLeft: aNumber
^ self costume renderedMorph move: aNumber negated @ 0! !

!CartesianGraphPlayer methodsFor: 'commands' stamp: 'Richo 4/15/2010 12:45'!
moveRight: aNumber
^ self costume renderedMorph move: aNumber @ 0! !

!CartesianGraphPlayer methodsFor: 'commands' stamp: 'Richo 4/15/2010 12:45'!
moveUp: aNumber
^ self costume renderedMorph move: 0 @ aNumber! !

!CartesianGraphPlayer methodsFor: 'commands' stamp: 'Richo 4/23/2010 15:22'!
move: aPoint
| p |
aPoint isString 
ifTrue: [p := aPoint subStrings: {$@}. p := p first asNumber @ p second asNumber ]
ifFalse: [p := aPoint].

^ self costume renderedMorph move: p! !

!CartesianGraphPlayer methodsFor: 'commands' stamp: 'Richo 4/15/2010 12:45'!
restoreDefaultColors
^ self costume renderedMorph restoreDefaultColors.! !

!CartesianGraphPlayer methodsFor: 'commands' stamp: 'Richo 4/15/2010 12:45'!
restoreDefaultGrids
^ self costume renderedMorph restoreDefaultGrids.! !

!CartesianGraphPlayer methodsFor: 'commands' stamp: 'Richo 4/15/2010 12:45'!
restoreInitialPosition
^ self costume renderedMorph restoreInitialPosition ! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setAxisColor: aColor
^ self costume renderedMorph axisColor: aColor! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 15:42'!
setBackgroundColor: aColor
^ self costume renderedMorph backgroundColor: aColor! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setMajorGridColor: aColor
^ self costume renderedMorph majorGridColor: aColor! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setMinorGridColor: aColor
^ self costume renderedMorph minorGridColor: aColor! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setShowCursorPosition: aBoolean
^ self costume renderedMorph showCursorPosition: aBoolean! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/23/2010 17:48'!
setXLegendSpacing: anInteger
| morph |
morph := self costume renderedMorph.
morph legendSpacing: ((anInteger max: 0) rounded  @ morph legendSpacing y)! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setXMajorGrid: aNumber
| morph |
morph := self costume renderedMorph.
morph majorGrid: (aNumber max: 0) rounded @ morph majorGrid y.! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setXMax: aNumber
| morph |
morph :=  self costume renderedMorph.
[morph cartesianBounds: (morph cartesianBounds origin corner: (aNumber @ morph cartesianBounds corner y))]
	on: Error do: [:err| self inform: err description].! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setXMinorGrid: aNumber
| morph |
morph := self costume renderedMorph.
morph minorGrid: (aNumber max: 0) rounded @ morph minorGrid y.! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setXMin: aNumber
| morph |
morph :=  self costume renderedMorph.
[morph cartesianBounds: ((aNumber @ morph cartesianBounds origin y) corner: morph cartesianBounds corner)]
	on: Error do: [:err| self inform: err description]..! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/23/2010 17:47'!
setYLegendSpacing: anInteger
| morph |
morph := self costume renderedMorph.
morph legendSpacing: (morph legendSpacing x @ (anInteger max: 0) rounded )! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setYMajorGrid: aNumber
| morph |
morph := self costume renderedMorph.
morph majorGrid: morph majorGrid x @ ((aNumber max: 0) rounded).! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setYMax: aNumber
| morph |
morph :=  self costume renderedMorph.
[morph cartesianBounds: (morph cartesianBounds origin corner: (morph cartesianBounds corner x @ aNumber))]
	on: Error do: [:err| self inform: err description]..! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setYMinorGrid: aNumber
| morph |
morph := self costume renderedMorph.
morph minorGrid: morph minorGrid x @ ((aNumber max: 0) rounded).! !

!CartesianGraphPlayer methodsFor: 'slots' stamp: 'Richo 4/15/2010 12:45'!
setYMin: aNumber
| morph |
morph :=  self costume renderedMorph.
[morph cartesianBounds: ((morph cartesianBounds origin x @ aNumber) corner: morph cartesianBounds corner)]
	on: Error do: [:err| self inform: err description]..! !

CartesianGraphPlayer subclass: #DataGraphPlayer
	instanceVariableNames: 'pointIndex vectorIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

DataGraphPlayer subclass: #BarGraphPlayer
	instanceVariableNames: 'barIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!BarGraphPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/23/2010 17:37'!
isUniClass
	^ self ~~ self officialClass! !

!BarGraphPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/23/2010 17:37'!
officialClass
	^ BarGraphPlayer! !

!BarGraphPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/23/2010 17:37'!
wantsChangeSetLogging
	^ (self == self officialClass
			or: [(self name beginsWith: self officialClass asString) not])
		or: [Preferences universalTiles]! !

!BarGraphPlayer methodsFor: 'commands-bars' stamp: 'Richo 4/25/2010 16:58'!
addBar
	self costume renderedMorph addBar: (($A to: $Z) at: self getBarCount + 1 ifAbsent: [$#]) asString -> 1.
	barIndex := self getBarCount! !

!BarGraphPlayer methodsFor: 'accessing' stamp: 'Richo 4/24/2010 13:27'!
currentBar
^self costume renderedMorph barAt: barIndex ifAbsent: [nil]! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 4/24/2010 13:23'!
getBarColor
^ [self costume renderedMorph barColorAt: barIndex] on: Error do: [Color black]! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 4/24/2010 13:23'!
getBarCount
^ self costume renderedMorph barCount! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 4/24/2010 13:24'!
getBarIndex
barIndex > self getBarCount ifTrue: [barIndex := self getBarCount].
^ barIndex! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 4/24/2010 13:26'!
getBarLabel
| current |
current := self currentBar.
^current isNil ifTrue: [nil] ifFalse: [current key]! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 4/24/2010 13:24'!
getBarValue
| current |
current := self currentBar.
^current isNil ifTrue: [nil] ifFalse: [current value]! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 5/21/2010 16:46'!
getDistanceBetweenBars 
^ self costume renderedMorph distanceBetweenBars! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 5/18/2010 10:53'!
getGraphDirection
^ self costume renderedMorph graphDirection! !

!BarGraphPlayer methodsFor: 'slots' stamp: 'Richo 5/21/2010 17:04'!
getLegendSpacing
^ self costume renderedMorph legendSpacing! !

!BarGraphPlayer methodsFor: 'slots' stamp: 'Richo 5/21/2010 16:57'!
getMajorGrid
^ self costume renderedMorph majorGrid! !

!BarGraphPlayer methodsFor: 'slots' stamp: 'Richo 5/21/2010 17:05'!
getMax
^self costume renderedMorph range last! !

!BarGraphPlayer methodsFor: 'slots' stamp: 'Richo 5/21/2010 17:05'!
getMin
^self costume renderedMorph range first! !

!BarGraphPlayer methodsFor: 'slots' stamp: 'Richo 5/21/2010 16:57'!
getMinorGrid
^ self costume renderedMorph minorGrid! !

!BarGraphPlayer methodsFor: 'initialize-release' stamp: 'Richo 4/23/2010 17:38'!
initialize
super initialize.
barIndex := 0.
! !

!BarGraphPlayer methodsFor: 'commands' stamp: 'Richo 4/23/2010 17:43'!
move: aNumber
^ self costume renderedMorph move: aNumber! !

!BarGraphPlayer methodsFor: 'commands-bars' stamp: 'Richo 5/13/2010 18:47'!
openBarTable
 self costume renderedMorph openBarTable! !

!BarGraphPlayer methodsFor: 'commands-bars' stamp: 'Richo 4/24/2010 13:22'!
removeAllBars
barIndex := 0.
^ self costume renderedMorph removeAllBars! !

!BarGraphPlayer methodsFor: 'commands-bars' stamp: 'Richo 4/24/2010 13:22'!
removeBarAt: aNumber
(aNumber > self getBarCount or: [aNumber <= 0])
	ifTrue: [^self].
self costume renderedMorph removeBarAt: aNumber.
barIndex = aNumber ifTrue: [self setBarIndex: barIndex - 1]! !

!BarGraphPlayer methodsFor: 'commands-bars' stamp: 'Richo 4/24/2010 13:23'!
removeCurrentBar
barIndex = 0 ifTrue: [^self].
self costume renderedMorph removeBarAt: barIndex.
self setBarIndex: barIndex - 1! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 4/24/2010 13:24'!
setBarColor: aColor
 [self costume renderedMorph barColorAt: barIndex put: aColor] on: Error do: []! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 4/24/2010 13:25'!
setBarIndex: aNumber
self getBarCount = 0 
	ifTrue: [barIndex := 0. ^self].
(aNumber > self getBarCount)
	ifTrue: [barIndex := 1.^self].
aNumber < 1
	ifTrue: [barIndex := self getBarCount.^self].
barIndex := aNumber.! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 4/24/2010 22:53'!
setBarLabel: aString
| current |
current := self currentBar.
current isNil ifTrue: [^self].
self costume renderedMorph barAt: barIndex put:  (aString -> current value)! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 4/24/2010 13:26'!
setBarValue: aNumber
| current |
current := self currentBar.
current isNil ifTrue: [^self].
self costume renderedMorph barAt: barIndex put:  (current key -> aNumber)! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 5/21/2010 16:46'!
setDistanceBetweenBars: aNumber
^ self costume renderedMorph distanceBetweenBars: aNumber! !

!BarGraphPlayer methodsFor: 'slots-bars' stamp: 'Richo 5/18/2010 10:54'!
setGraphDirection: aSymbol
	self costume renderedMorph graphDirection: aSymbol! !

!BarGraphPlayer methodsFor: 'slots' stamp: 'Richo 5/21/2010 17:04'!
setLegendSpacing: aNumber
^ self costume renderedMorph legendSpacing: (aNumber max: 0) rounded! !

!BarGraphPlayer methodsFor: 'slots' stamp: 'Richo 5/21/2010 16:57'!
setMajorGrid: aNumber
^ self costume renderedMorph majorGrid: (aNumber max: 0) rounded! !

!BarGraphPlayer methodsFor: 'slots' stamp: 'Richo 5/21/2010 17:05'!
setMax: aNumber
| morph |
morph := self costume renderedMorph.
morph range: (morph range first to: aNumber)! !

!BarGraphPlayer methodsFor: 'slots' stamp: 'Richo 5/21/2010 16:57'!
setMinorGrid: aNumber
^ self costume renderedMorph minorGrid: (aNumber max: 0) rounded! !

!BarGraphPlayer methodsFor: 'slots' stamp: 'Richo 5/21/2010 17:05'!
setMin: aNumber
| morph |
morph := self costume renderedMorph.
morph range: (aNumber to: morph range last)! !

!DataGraphPlayer class methodsFor: 'change set logging' stamp: 'Richo 5/21/2010 08:54'!
isUniClass
	^ self ~~ self officialClass! !

!DataGraphPlayer class methodsFor: 'change set logging' stamp: 'Richo 5/21/2010 08:54'!
officialClass
	^ DataGraphPlayer! !

!DataGraphPlayer class methodsFor: 'change set logging' stamp: 'Richo 5/21/2010 08:54'!
wantsChangeSetLogging
	^ (self == self officialClass
			or: [(self name beginsWith: self officialClass asString) not])
		or: [Preferences universalTiles]! !

!DataGraphPlayer methodsFor: 'commands-points' stamp: 'Richo 5/21/2010 08:49'!
addPoint
	self costume renderedMorph addPoint: (0@0).
	pointIndex := self getPointCount! !

!DataGraphPlayer methodsFor: 'commands-vectors' stamp: 'Richo 5/21/2010 08:49'!
addVector
	self costume renderedMorph addVector: ((0@0) to: (0@0)).
	vectorIndex := self getVectorCount! !

!DataGraphPlayer methodsFor: 'accessing' stamp: 'Richo 5/21/2010 08:50'!
currentPoint
^self costume renderedMorph pointAt: pointIndex ifAbsent: [nil]! !

!DataGraphPlayer methodsFor: 'accessing' stamp: 'Richo 5/21/2010 08:50'!
currentVector
^self costume renderedMorph vectorAt: vectorIndex ifAbsent: [nil]! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:50'!
getInitialPointX
| current |
current := self currentVector.
^current isNil ifTrue: [nil] ifFalse: [current initialPoint x]! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:50'!
getInitialPointY
| current |
current := self currentVector.
^current isNil ifTrue: [nil] ifFalse: [current initialPoint y]! !

!DataGraphPlayer methodsFor: 'slots-points' stamp: 'Richo 5/21/2010 09:15'!
getPlayerAtPointIndex
	| morph |
	morph := self costume renderedMorph pointMorphAt: pointIndex.
	morph isNil ifTrue: [^ self presenter standardPlayer].
	^morph assuredPlayer! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 09:16'!
getPlayerAtVectorIndex
	| morph |
	morph := self costume renderedMorph vectorMorphAt: vectorIndex.
	morph isNil ifTrue: [^ self presenter standardPlayer].
	^morph assuredPlayer! !

!DataGraphPlayer methodsFor: 'slots-points' stamp: 'Richo 5/21/2010 08:52'!
getPointColor
^ [self costume renderedMorph pointColorAt: pointIndex] on: Error do: [Color black]! !

!DataGraphPlayer methodsFor: 'slots-points' stamp: 'Richo 5/21/2010 08:52'!
getPointCount
^ self costume renderedMorph pointCount! !

!DataGraphPlayer methodsFor: 'slots-points' stamp: 'Richo 5/21/2010 08:52'!
getPointIndex
pointIndex > self getPointCount ifTrue: [pointIndex := self getPointCount].
^ pointIndex! !

!DataGraphPlayer methodsFor: 'slots-points' stamp: 'Richo 5/21/2010 08:52'!
getPointX
| current |
current := self currentPoint.
^current isNil ifTrue: [nil] ifFalse: [current x]! !

!DataGraphPlayer methodsFor: 'slots-points' stamp: 'Richo 5/21/2010 08:52'!
getPointY
| current |
current := self currentPoint.
^current isNil ifTrue: [nil] ifFalse: [current y]! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:50'!
getTerminalPointX
| current |
current := self currentVector.
^current isNil ifTrue: [nil] ifFalse: [current terminalPoint x]! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:50'!
getTerminalPointY
| current |
current := self currentVector.
^current isNil ifTrue: [nil] ifFalse: [current terminalPoint y]! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
getVectorAngle
| current |
current := self currentVector.
^current isNil ifTrue: [nil] ifFalse: [current angle]! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
getVectorColor
^ [self costume renderedMorph vectorColorAt: vectorIndex] on: Error do: [Color black]! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
getVectorCount
^self costume renderedMorph vectorCount! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
getVectorIndex
vectorIndex > self getVectorCount ifTrue: [vectorIndex := self getVectorCount].
^ vectorIndex! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
getVectorMagnitude
| current |
current := self currentVector.
^current isNil ifTrue: [nil] ifFalse: [current magnitude]! !

!DataGraphPlayer methodsFor: 'initialize' stamp: 'Richo 5/21/2010 08:53'!
initialize
super initialize.
pointIndex := vectorIndex :=  0.
! !

!DataGraphPlayer methodsFor: 'commands-points' stamp: 'Richo 5/21/2010 08:49'!
openPointTable
^ self costume renderedMorph openPointTable! !

!DataGraphPlayer methodsFor: 'commands-points' stamp: 'Richo 5/21/2010 08:49'!
removeAllPoints
pointIndex := 0.
^ self costume renderedMorph removeAllPoints! !

!DataGraphPlayer methodsFor: 'commands-vectors' stamp: 'Richo 5/21/2010 08:49'!
removeAllVectors.
vectorIndex := 0.
^ self costume renderedMorph removeAllVectors! !

!DataGraphPlayer methodsFor: 'commands-points' stamp: 'Richo 5/21/2010 08:49'!
removeCurrentPoint
pointIndex = 0 ifTrue: [^self].
self costume renderedMorph removePointAt: pointIndex.
self setPointIndex: pointIndex - 1! !

!DataGraphPlayer methodsFor: 'commands-vectors' stamp: 'Richo 5/21/2010 08:50'!
removeCurrentVector
vectorIndex = 0 ifTrue: [^self].
self costume renderedMorph removeVectorAt: vectorIndex.
self setVectorIndex: vectorIndex - 1! !

!DataGraphPlayer methodsFor: 'commands-points' stamp: 'Richo 5/21/2010 08:49'!
removePointAt: aNumber
(aNumber > self getPointCount or: [aNumber <= 0])
	ifTrue: [^self].
self costume renderedMorph removePointAt: aNumber.
pointIndex = aNumber ifTrue: [self setPointIndex: pointIndex - 1]! !

!DataGraphPlayer methodsFor: 'commands-vectors' stamp: 'Richo 5/21/2010 08:50'!
removeVectorAt: aNumber
(aNumber > self getVectorCount or: [aNumber <= 0])
	ifTrue: [^self].
self costume renderedMorph removeVectorAt: aNumber.
vectorIndex = aNumber ifTrue: [self setVectorIndex: vectorIndex - 1]! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
setInitialPointX: aNumber
| current |
current := self currentVector.
current isNil ifTrue: [^self].
self costume renderedMorph vectorAt: vectorIndex put: (Vector initialPoint: (aNumber @ current initialPoint y) angle: current angle magnitude: current magnitude)
! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
setInitialPointY: aNumber
| current |
current := self currentVector.
current isNil ifTrue: [^self].
self costume renderedMorph vectorAt: vectorIndex put: (Vector initialPoint: (current initialPoint x @ aNumber) angle: current angle magnitude: current magnitude)
! !

!DataGraphPlayer methodsFor: 'slots-points' stamp: 'Richo 5/21/2010 08:53'!
setPointColor: aColor
 [self costume renderedMorph pointColorAt: pointIndex put: aColor] on: Error do: []! !

!DataGraphPlayer methodsFor: 'slots-points' stamp: 'Richo 5/21/2010 08:53'!
setPointIndex: aNumber
self getPointCount = 0 
	ifTrue: [pointIndex := 0. ^self].
(aNumber > self getPointCount)
	ifTrue: [pointIndex := 1.^self].
aNumber < 1
	ifTrue: [pointIndex := self getPointCount.^self].
pointIndex := aNumber.! !

!DataGraphPlayer methodsFor: 'slots-points' stamp: 'Richo 5/21/2010 08:53'!
setPointX: aNumber
| current |
current := self currentPoint.
current isNil ifTrue: [^self].
self costume renderedMorph pointAt: pointIndex put:  aNumber @ current y! !

!DataGraphPlayer methodsFor: 'slots-points' stamp: 'Richo 5/21/2010 08:53'!
setPointY: aNumber
| current |
current := self currentPoint.
current isNil ifTrue: [^self].
self costume renderedMorph pointAt: pointIndex put:  current x @ aNumber! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
setTerminalPointX: aNumber
| current |
current := self currentVector.
current isNil ifTrue: [^self].
self costume renderedMorph vectorAt: vectorIndex put: (current initialPoint to: (aNumber @ current terminalPoint y))! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
setTerminalPointY: aNumber
| current |
current := self currentVector.
current isNil ifTrue: [^self].
self costume renderedMorph vectorAt: vectorIndex put: (current initialPoint to: (current terminalPoint x @ aNumber))! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
setVectorAngle: aNumber
| current |
current := self currentVector.
current isNil ifTrue: [^self].
self costume renderedMorph vectorAt: vectorIndex put: (Vector initialPoint: current initialPoint angle: aNumber magnitude: current magnitude)! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:51'!
setVectorColor: aColor
 [self costume renderedMorph vectorColorAt: vectorIndex put: aColor] on: Error do: []! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:52'!
setVectorIndex: aNumber
self getVectorCount = 0 
	ifTrue: [vectorIndex := 0. ^self].
(aNumber > self getVectorCount)
	ifTrue: [vectorIndex := 1.^self].
aNumber < 1
	ifTrue: [vectorIndex := self getVectorCount.^self].
vectorIndex := aNumber.! !

!DataGraphPlayer methodsFor: 'slots-vectors' stamp: 'Richo 5/21/2010 08:52'!
setVectorMagnitude: aNumber
| current |
current := self currentVector.
current isNil ifTrue: [^self].
self costume renderedMorph vectorAt: vectorIndex put: (Vector initialPoint: current initialPoint angle: current angle magnitude: aNumber)! !

!Player methodsFor: '*GSoC' stamp: 'Richo 6/17/2010 00:02'!
getBubble
^[self costume renderedMorph bubble assuredPlayer] on: Error do: [self presenter standardPlayer]! !

!Player methodsFor: '*GSoC' stamp: 'Richo 6/13/2010 16:09'!
say: aString
self costume renderedMorph say: aString! !

!Player methodsFor: '*GSoC' stamp: 'Richo 6/13/2010 16:09'!
think: aString
self costume renderedMorph think: aString! !

Player subclass: #PointPlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!PointPlayer commentStamp: 'Richo 4/17/2010 21:07' prior: 0!
This class is needed to add Etoys scripting capabitlities to PointMorph. It's actually very useless.!

!PointPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/17/2010 20:38'!
isUniClass
	^ self ~~ self officialClass! !

!PointPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/17/2010 20:38'!
officialClass
	^ PointPlayer! !

!PointPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/17/2010 20:38'!
wantsChangeSetLogging
	^ (self == self officialClass
			or: [(self name beginsWith: self officialClass asString) not])
		or: [Preferences universalTiles]! !

!PointPlayer methodsFor: 'slots' stamp: 'Richo 4/21/2010 22:49'!
getPointColor
^ self costume renderedMorph color! !

!PointPlayer methodsFor: 'slots' stamp: 'Richo 4/20/2010 23:51'!
getPointX
^ self costume renderedMorph point x! !

!PointPlayer methodsFor: 'slots' stamp: 'Richo 4/20/2010 23:51'!
getPointY
^ self costume renderedMorph point y! !

!PointPlayer methodsFor: 'commands' stamp: 'Richo 4/26/2010 22:56'!
remove
^ self costume renderedMorph remove! !

!PointPlayer methodsFor: 'slots' stamp: 'Richo 4/21/2010 22:48'!
setPointColor: aColor
^ self costume renderedMorph color: aColor! !

!PointPlayer methodsFor: 'slots' stamp: 'Richo 4/23/2010 16:37'!
setPointX: aNumber
| morph |
morph := self costume renderedMorph.
morph point: aNumber @ morph point y.! !

!PointPlayer methodsFor: 'slots' stamp: 'Richo 4/23/2010 16:37'!
setPointY: aNumber
| morph |
morph := self costume renderedMorph.
morph point: morph point x @ aNumber! !

Player subclass: #PointTablePlayer
	instanceVariableNames: 'pointIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!PointTablePlayer commentStamp: 'Richo 4/17/2010 23:34' prior: 0!
This class is needed to add Etoys scripting capabitlities to PointTableMorph. It lets the user go through the list of points, see how many rows are filled, how many are empty, and so on.!

!PointTablePlayer class methodsFor: 'change set logging' stamp: 'Richo 4/17/2010 20:39'!
isUniClass
	^ self ~~ self officialClass! !

!PointTablePlayer class methodsFor: 'change set logging' stamp: 'Richo 4/17/2010 20:39'!
officialClass
	^ PointTablePlayer! !

!PointTablePlayer class methodsFor: 'change set logging' stamp: 'Richo 4/17/2010 20:40'!
wantsChangeSetLogging
	^ (self == self officialClass
			or: [(self name beginsWith: self officialClass asString) not])
		or: [Preferences universalTiles]! !

!PointTablePlayer methodsFor: 'commands' stamp: 'Richo 4/16/2010 23:02'!
addPoint
^ self costume renderedMorph addPoint! !

!PointTablePlayer methodsFor: 'commands' stamp: 'Richo 4/16/2010 23:02'!
apply
^ self costume renderedMorph apply! !

!PointTablePlayer methodsFor: 'slots' stamp: 'Richo 4/25/2010 12:53'!
getPointCount
^( self costume renderedMorph list submorphs select: [:each| each isAlignmentMorph and: [each submorphs first isTextMorph]]) size! !

!PointTablePlayer methodsFor: 'slots' stamp: 'Richo 4/26/2010 22:42'!
getPointIndex
pointIndex > self getPointCount ifTrue: [pointIndex := self getPointCount].
^ pointIndex! !

!PointTablePlayer methodsFor: 'slots' stamp: 'Richo 4/16/2010 23:05'!
getPointsFilled
^ self costume renderedMorph points size! !

!PointTablePlayer methodsFor: 'slots' stamp: 'Richo 5/13/2010 19:05'!
getPointX
| morph count |
morph := self costume renderedMorph.
count := 0.
morph list submorphs do: [:each|
	(each isAlignmentMorph and: [ each submorphs first isTextMorph])
	 ifTrue: [
		count := count + 1.
		count = pointIndex ifTrue: [
			^[each submorphs first contents asString asNumber] on: Error do: [0].
			]			
		]
	]
! !

!PointTablePlayer methodsFor: 'slots' stamp: 'Richo 5/13/2010 19:05'!
getPointY
| morph count |
morph := self costume renderedMorph.
count := 0.
morph list submorphs do: [:each|
	(each isAlignmentMorph and: [ each submorphs first isTextMorph])
	 ifTrue: [
		count := count + 1.
		count = pointIndex ifTrue: [
			^[each submorphs second contents asString asNumber] on: Error do: [0].
			]			
		]
	]! !

!PointTablePlayer methodsFor: 'initialize-release' stamp: 'Richo 4/16/2010 23:10'!
initialize
super initialize.
pointIndex := 1.! !

!PointTablePlayer methodsFor: 'commands' stamp: 'Richo 4/27/2010 12:44'!
removePoint
^ self costume renderedMorph removePoint! !

!PointTablePlayer methodsFor: 'slots' stamp: 'Richo 4/26/2010 22:43'!
setPointIndex: aNumber 
self getPointCount = 0 
	ifTrue: [pointIndex := 0. ^self].
(aNumber > self getPointCount)
	ifTrue: [pointIndex := 1.^self].
aNumber < 1
	ifTrue: [pointIndex := self getPointCount.^self].
pointIndex := aNumber.! !

!PointTablePlayer methodsFor: 'slots' stamp: 'Richo 4/25/2010 12:53'!
setPointX: aNumber
| morph count |
morph := self costume renderedMorph.
count := 0.
morph list submorphs do: [:each|
	(each isAlignmentMorph and: [ each submorphs first isTextMorph])
	 ifTrue: [
		count := count + 1.
		count = pointIndex ifTrue: [
			each submorphs first contents: aNumber asString.
			^self
			]			
		]
	]! !

!PointTablePlayer methodsFor: 'slots' stamp: 'Richo 4/25/2010 12:53'!
setPointY: aNumber
| morph count |
morph := self costume renderedMorph.
count := 0.
morph list submorphs do: [:each|
	(each isAlignmentMorph and: [ each submorphs first isTextMorph])
	 ifTrue: [
		count := count + 1.
		count = pointIndex ifTrue: [
			each submorphs second contents: aNumber asString.
			^self
			]			
		]
	]! !

Player subclass: #SpeechBubblePlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-SpeechBubble'!

!SpeechBubblePlayer methodsFor: 'as yet unclassified' stamp: 'Richo 6/16/2010 21:47'!
attachTo: aPlayer
self costume renderedMorph target: aPlayer costume renderedMorph
! !

!SpeechBubblePlayer methodsFor: 'as yet unclassified' stamp: 'Richo 6/16/2010 21:55'!
getAttachment
	^ [self costume renderedMorph target assuredPlayer]
		on: Error
		do: [self presenter standardPlayer] ! !

!SpeechBubblePlayer methodsFor: 'as yet unclassified' stamp: 'Richo 6/16/2010 21:46'!
stopAttaching
	self costume renderedMorph target: nil! !

Player subclass: #SpreadsheetPlayer
	instanceVariableNames: 'rowIndex columnIndex fileName separator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-IO'!

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:07'!
getColumnIndex
columnIndex > self getTotalColumns ifTrue: [columnIndex := self getTotalColumns].
^ columnIndex! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/17/2010 22:34'!
getCsvFileName
	^fileName! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/18/2010 11:54'!
getCsvSeparator
^separator asString! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:24'!
getCurrentCharacters
^self costume renderedMorph cellValueAt: self getRowIndex @ self getColumnIndex! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:24'!
getCurrentNumericValue
^self costume renderedMorph cellNumericValueAt: self getRowIndex @ self getColumnIndex! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:07'!
getRowIndex
rowIndex > self getTotalRows ifTrue: [rowIndex := self getTotalRows].
^ rowIndex! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 10:59'!
getTotalColumns
^self costume renderedMorph numberOfColumns! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 10:59'!
getTotalRows
^self costume renderedMorph numberOfRows! !

!SpreadsheetPlayer methodsFor: 'initialize' stamp: 'Richo 6/18/2010 11:06'!
initialize
super initialize.
rowIndex := 1.
columnIndex := 1.
separator := $,.
fileName := 'fileName.csv'! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/18/2010 12:39'!
loadFile
[self costume renderedMorph openCSVFile: fileName separator: separator] on: Error do: [self inform: 'File error']! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/18/2010 14:28'!
saveFile
[self costume renderedMorph saveCSVFile: fileName separator: separator] on: Error do: [self inform: 'File error']! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:08'!
setColumnIndex: aNumber 
(aNumber > self getTotalColumns)
	ifTrue: [columnIndex := 1.^self].
aNumber < 1
	ifTrue: [columnIndex := self getTotalColumns.^self].
columnIndex := aNumber.! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/17/2010 22:36'!
setCsvFileName: aString
	fileName := aString! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/18/2010 11:55'!
setCsvSeparator: aString
separator := aString first! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:25'!
setCurrentCharacters: aString
^self costume renderedMorph cellValueAt: self getRowIndex @ self getColumnIndex put: aString! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:25'!
setCurrentNumericValue: aNumber
^self costume renderedMorph cellNumericValueAt: self getRowIndex @ self getColumnIndex put: aNumber! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:08'!
setRowIndex: aNumber 
(aNumber > self getTotalRows)
	ifTrue: [rowIndex := 1.^self].
aNumber < 1
	ifTrue: [rowIndex := self getTotalRows.^self].
rowIndex := aNumber.! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 10:59'!
setTotalColumns: aNumber
^self costume renderedMorph numberOfColumns: (aNumber max: 1)! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 10:59'!
setTotalRows: aNumber
^self costume renderedMorph numberOfRows: (aNumber max: 1)! !

Player subclass: #VectorPlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!VectorPlayer commentStamp: 'Richo 4/18/2010 00:26' prior: 0!
This class is needed to add Etoys scripting capabitlities to VectorMorph. It allows the user to change any of the vector's parameters.!

!VectorPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/17/2010 20:40'!
isUniClass
	^ self ~~ self officialClass! !

!VectorPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/17/2010 20:40'!
officialClass
	^ VectorPlayer! !

!VectorPlayer class methodsFor: 'change set logging' stamp: 'Richo 4/17/2010 20:40'!
wantsChangeSetLogging
	^ (self == self officialClass
			or: [(self name beginsWith: self officialClass asString) not])
		or: [Preferences universalTiles]! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:24'!
getInitialPointX
^ self costume renderedMorph vector initialPoint x! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:24'!
getInitialPointY
^ self costume renderedMorph vector initialPoint y! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:24'!
getTerminalPointX
^ self costume renderedMorph vector terminalPoint x! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:24'!
getTerminalPointY
^ self costume renderedMorph vector terminalPoint y! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:24'!
getVectorAngle
^ self costume renderedMorph vector angle! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:24'!
getVectorColor
^ self costume renderedMorph color! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:25'!
getVectorMagnitude
^ self costume renderedMorph vector magnitude! !

!VectorPlayer methodsFor: 'commands' stamp: 'Richo 4/17/2010 00:23'!
remove
^ self costume renderedMorph remove! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:40'!
setInitialPointX: aNumber
| morph vector |
morph := self costume renderedMorph.
vector := morph vector.
morph vector: (Vector initialPoint: (aNumber @ vector initialPoint y) angle: vector angle magnitude: vector magnitude)
! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:40'!
setInitialPointY: aNumber
| morph vector |
morph := self costume renderedMorph.
vector := morph vector.
morph vector: (Vector initialPoint: (vector initialPoint x @ aNumber) angle: vector angle magnitude: vector magnitude)
! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:41'!
setTerminalPointX: aNumber
| morph vector |
morph := self costume renderedMorph.
vector := morph vector.
morph vector: (vector initialPoint to: (aNumber @ vector terminalPoint y))! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:42'!
setTerminalPointY: aNumber
| morph vector |
morph := self costume renderedMorph.
vector := morph vector.
morph vector: (vector initialPoint to: (vector terminalPoint x @ aNumber))! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:43'!
setVectorAngle: aNumber
| morph vector |
morph := self costume renderedMorph.
vector := morph vector.
morph vector: (Vector initialPoint: vector initialPoint angle: aNumber magnitude: vector magnitude)! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:43'!
setVectorColor: aColor
^ self costume renderedMorph color: aColor! !

!VectorPlayer methodsFor: 'slots' stamp: 'Richo 4/17/2010 00:44'!
setVectorMagnitude: aNumber
| morph vector |
morph := self costume renderedMorph.
vector := morph vector.
morph vector: (Vector initialPoint: vector initialPoint angle: vector angle magnitude: aNumber)! !

!Point methodsFor: '*GSoC' stamp: 'Richo 4/13/2010 08:45'!
to: aPoint 
	^Vector initialPoint: self terminalPoint: aPoint! !

PasteUpMorph subclass: #CartesianGraphMorph
	instanceVariableNames: 'cartesianBounds minorGrid majorGrid legendSpacing backgroundColor minorGridColor majorGridColor axisColor verticalAxis horizontalAxis legends grids gridForm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!CartesianGraphMorph commentStamp: 'Richo 5/21/2010 08:59' prior: 0!
This class is responsible of drawing a cartesian coordinate system.
You can change its grid, x and y range, legend spacing and colors using the following methods:
	#majorGrid:
	#minorGrid:
	#cartesianBounds:
	#legendSpacing:
	#axisColor:
	#backgroundColor:
	#majorGridColor:
	#minorGridColor:!

CartesianGraphMorph subclass: #BarGraphMorph
	instanceVariableNames: 'barMorphs distanceBetweenBars graphType graphDirection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!BarGraphMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/18/2010 11:01'!
additionsToViewerCategoryPlotBars
^#(#'plot - bars' (
#(slot graphDirection '' GraphDirection  readWrite Player getGraphDirection Player setGraphDirection:  )
#(slot barIndex '' Number  readWrite Player getBarIndex Player setBarIndex:  )
#(command removeBarAt: '' Number )
#(command removeAllBars '')
#(command removeCurrentBar '')
#(command addBar '')
#(slot barValue '' Number readWrite Player getBarValue Player setBarValue:  )
#(slot barLabel '' String readWrite Player getBarLabel Player setBarLabel:  )
#(command openBarTable '')
#(slot barCount '' Number readOnly Player getBarCount Player unused  )
#(slot barColor '' Color readWrite Player getBarColor Player setBarColor:  )
#(slot distanceBetweenBars '' Number readWrite Player getDistanceBetweenBars Player setDistanceBetweenBars:  )
))! !

!BarGraphMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/21/2010 16:54'!
additionsToViewerCategoryPlotGrid
"This is just to override the #'plot - grid' category (notice the space after grid)"
^#(#'plot - grid ' (
#(slot majorGrid '' Number readWrite Player getMajorGrid Player setMajorGrid:  )
#(slot minorGrid '' Number readWrite Player getMinorGrid Player setMinorGrid:  )
#(command restoreDefaultGrids '' )
))! !

!BarGraphMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/21/2010 17:04'!
additionsToViewerCategoryPlotLegends
"This is just to override the #'plot - legends' category (notice the space after grid)"
^#(#'plot - legends ' (
#(slot spacing '' Number readWrite Player getLegendSpacing Player setLegendSpacing:  )
))! !

!BarGraphMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/21/2010 17:04'!
additionsToViewerCategoryPlotRange
^#(#'plot - range' (
#(command move: '' Number)
#(command restoreInitialPosition '')
#(slot max '' Number readWrite Player getMax Player setMax:  )
#(slot min '' Number readWrite Player getMin Player setMin:  )
))! !

!BarGraphMorph class methodsFor: 'parts bin' stamp: 'Richo 4/23/2010 17:30'!
defaultNameStemForInstances
	^ 'Bar Graph' ! !

!BarGraphMorph class methodsFor: 'parts bin' stamp: 'Richo 4/23/2010 17:30'!
descriptionForPartsBin
	^ self
		partName: 'Bar Graph'
		categories: #('GSoC' )
		documentation: ''
! !

!BarGraphMorph class methodsFor: 'new-morph participation' stamp: 'Richo 4/23/2010 17:30'!
includeInNewMorphMenu
	"Return true for all classes that can be instantiated from the menu"
	^ true! !

!BarGraphMorph class methodsFor: 'class initialization' stamp: 'Richo 5/18/2010 11:02'!
initialize
"self initialize"
super initialize.
Vocabulary addStandardVocabulary: (SymbolListType new vocabularyName: #GraphDirection;
			 symbols: #(#horizontal #vertical )).! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 4/24/2010 14:51'!
addBar: anAssociation
| m |
m := BarMorph graph: self bar: anAssociation index: self barCount + 1.
self addMorph: m.
barMorphs add: m.
self changed.! !

!BarGraphMorph methodsFor: 'menus' stamp: 'Richo 4/26/2010 23:10'!
addCustomMenuItems: aMenu hand: aHandMorph
aMenu add: 'open bar table' target: self action: #openBarTable! !

!BarGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/21/2010 16:18'!
axisXPos
"Returns the horizontal position of the axis. It makes sure that the legends fit in this space"
| font |
font := TextStyle defaultFont.
^self left + ((font widthOfString: self range last ceiling asString) max: (font widthOfString: self range first floor asString)) + 5! !

!BarGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/21/2010 17:59'!
axisYPos
"Returns the vertical position of the axis. It makes sure that the legends fit in this space"
^self bottom - TextStyle defaultFont height - 5
! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 4/24/2010 13:28'!
barAt: index ifAbsent: aBlock
| m |
m := barMorphs at: index ifAbsent: [nil].
^m isNil ifTrue: [aBlock value] ifFalse: [m bar]
! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 4/24/2010 13:29'!
barAt: index put: anAssociation 
(barMorphs at: index ifAbsent: [^self error: 'Bar not found']) bar: anAssociation.! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 4/24/2010 22:05'!
barColorAt: index
^(barMorphs at: index) color! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 4/24/2010 22:05'!
barColorAt: index put: aColor
(barMorphs at: index) color: aColor! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 4/24/2010 09:45'!
barCount
^barMorphs size! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 4/26/2010 23:10'!
bars
^barMorphs collect: [:each| each bar]! !

!BarGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/21/2010 17:58'!
centerFor: anAssociation index: index
"Returns the correct center for a bar morph depending on its index and its value"
| x y |
self isVerticalGraph
ifTrue: [x := self axisXPos + (self distanceBetweenBars * index).
	y := (self cartesianYToPixel: anAssociation value / 2) rounded]
ifFalse: [x := (self cartesianXToPixel: anAssociation value / 2) rounded.
	y := self axisYPos - (self distanceBetweenBars * index)].
^x@y! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 4/24/2010 13:14'!
distanceBetweenBars
^distanceBetweenBars! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 4/24/2010 13:14'!
distanceBetweenBars: aNumber
distanceBetweenBars := aNumber.
self updateBars! !

!BarGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 16:06'!
drawGridForm
| aCanvas |
"For performance, I keep the background in a separate form and redraw it only when it's necessary (i.e. changing the gridding parameters or the extent of the morph)"
gridForm isNil ifTrue: [^self].
aCanvas := gridForm getCanvas.
aCanvas fillColor: backgroundColor .
self isVerticalGraph ifTrue: [self drawHorizontalGridOn: aCanvas] ifFalse: [self drawVerticalGridOn: aCanvas]! !

!BarGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/21/2010 17:58'!
extentFor: anAssociation
"Returns the correct extent for a bar morph depending on its value"
^self isVerticalGraph
	ifTrue: [30@((self cartesianYToRelativePixel: 0) - (self cartesianYToRelativePixel: anAssociation value)) abs]
	ifFalse: [((self cartesianXToRelativePixel: 0) - (self cartesianXToRelativePixel: anAssociation value)) abs @ 30]! !

!BarGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/21/2010 17:03'!
filterViewerCategoryDictionary: dict
	super filterViewerCategoryDictionary: dict.
	dict removeKey: #'plot - coordinate system' ifAbsent: [].
	dict removeKey: #'plot - grid' ifAbsent: [].
	dict removeKey: #'plot - legends' ifAbsent: [].! !

!BarGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/18/2010 10:56'!
graphDirection
^graphDirection! !

!BarGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/18/2010 10:56'!
graphDirection: aSymbol
"graphDirection must be either #vertical or #horizontal"
graphDirection := aSymbol.
self updateAll ! !

!BarGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/21/2010 18:00'!
initialize
	graphDirection := #horizontal.
	self initializeBars;
		 initializeDistanceBetweenBars.
	super initialize.! !

!BarGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 4/23/2010 23:46'!
initializeBars
barMorphs := OrderedCollection new.! !

!BarGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 4/24/2010 14:50'!
initializeDistanceBetweenBars
	distanceBetweenBars := 45
! !

!BarGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 4/24/2010 14:53'!
initializeToStandAlone
	"Answer an instance of the receiver suitable for placing in a parts bin for authors"
	
	self initialize.! !

!BarGraphMorph methodsFor: 'testing' stamp: 'Richo 5/18/2010 10:55'!
isVerticalGraph
^graphDirection = #vertical! !

!BarGraphMorph methodsFor: 'accessing legends' stamp: 'Richo 5/21/2010 17:02'!
legendSpacing
^self isVerticalGraph ifTrue: [super legendSpacing y] ifFalse: [super legendSpacing x]! !

!BarGraphMorph methodsFor: 'accessing legends' stamp: 'Richo 5/21/2010 17:01'!
legendSpacing: aNumber
super legendSpacing: aNumber@aNumber! !

!BarGraphMorph methodsFor: 'accessing grid' stamp: 'Richo 5/21/2010 16:56'!
majorGrid
^self isVerticalGraph ifTrue: [super majorGrid y] ifFalse: [super majorGrid x]! !

!BarGraphMorph methodsFor: 'accessing grid' stamp: 'Richo 5/21/2010 16:56'!
majorGrid: aNumber
super majorGrid: aNumber@aNumber! !

!BarGraphMorph methodsFor: 'accessing grid' stamp: 'Richo 5/21/2010 16:56'!
minorGrid
^self isVerticalGraph ifTrue: [super minorGrid y] ifFalse: [super minorGrid x]! !

!BarGraphMorph methodsFor: 'accessing grid' stamp: 'Richo 5/21/2010 16:56'!
minorGrid: aNumber
super minorGrid: aNumber@aNumber! !

!BarGraphMorph methodsFor: 'moving' stamp: 'Richo 5/21/2010 16:34'!
move: aNumber
self range: ((self range first + aNumber) to: (self range last + aNumber)).
self update! !

!BarGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 4/23/2010 17:35'!
newPlayerInstance
^BarGraphPlayer newUserInstance! !

!BarGraphMorph methodsFor: 'actions' stamp: 'Richo 4/26/2010 23:09'!
openBarTable
^(BarTableMorph on: self) openInHand! !

!BarGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 16:22'!
placeAxes
	| axisPos |
	self isVerticalGraph 
		ifTrue: [axisPos := self axisXPos @ (self cartesianYToPixel: 0).]
		ifFalse: [axisPos := (self cartesianXToPixel: 0) @ self axisYPos.].

	verticalAxis vertices at: 1 put: axisPos x @ self bottom.
	verticalAxis vertices at: 2 put: axisPos x @ self top.
	verticalAxis borderColor: axisColor.
	verticalAxis computeBounds.
	horizontalAxis vertices at: 1 put: self left @ axisPos y.
	horizontalAxis vertices at: 2 put: self right @ axisPos y.
	horizontalAxis borderColor: axisColor.
	horizontalAxis computeBounds! !

!BarGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 16:20'!
placeXLegends
self isVerticalGraph 
	ifTrue: [(legends at: #x) do: [:each | each delete].
		legends at: #x put: OrderedCollection new.
		^self].
super placeXLegends! !

!BarGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 17:17'!
placeYLegends
self isVerticalGraph 
	ifFalse: [(legends at: #y) do: [:each | each delete].
		legends at: #y put: OrderedCollection new.
		^self].
self placeYLegendsIncludeZero! !

!BarGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 17:16'!
placeYLegendsIncludeZero
"Almost like super>>#placeYLegends but this includes the zero"
| index yLegends legendPos legend |
index := 0.
yLegends := legends at: #y.
cartesianBounds top ceiling to: cartesianBounds bottom floor do: [:y | 
		| pos |
		pos := self cartesianYToPixel: y.
		(y isDivisibleBy: legendSpacing y)
			ifTrue: [
			legendPos := (self preferredPositionForYLegend: y asString at: pos).
			legendPos notNil ifTrue: [
				legend := yLegends at: (index := index + 1) ifAbsent: [yLegends add: StringMorph new].
				self addMorph: legend.
				legend contents: y asString; position: legendPos; color: axisColor]]].
yLegends isEmpty ifFalse: [
yLegends from: index + 1 to: yLegends size do: [:each| each delete].
yLegends removeLast: (yLegends size - index)]! !

!BarGraphMorph methodsFor: 'accessing range' stamp: 'Richo 5/21/2010 16:13'!
range
self isVerticalGraph
ifTrue: [^cartesianBounds top to: cartesianBounds bottom]
ifFalse: [^cartesianBounds left to: cartesianBounds right]
! !

!BarGraphMorph methodsFor: 'accessing range' stamp: 'Richo 5/21/2010 16:16'!
range: anInterval
self cartesianBounds: ((anInterval first @ anInterval first) corner: (anInterval last @ anInterval last)).! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 4/24/2010 22:51'!
removeAllBars
	barMorphs do: [:each| each delete].
	self initializeBars! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 5/13/2010 20:37'!
removeBarAt: index 
	| m |
	m := barMorphs at: index ifAbsent: [^self error: 'Bar not found'].
	m delete.
	barMorphs removeAt: index.
	self updateBars! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 5/13/2010 20:37'!
removeBarsFrom: start to: stop
| removed |
removed := OrderedCollection new.
start to: stop do: [:i|
	removed add: (barMorphs at: i) delete.
	].
barMorphs removeAllFoundIn: removed.
self updateBars.! !

!BarGraphMorph methodsFor: 'accessing bars' stamp: 'Richo 5/13/2010 20:37'!
removeBar: anAssoc
	| m |
	m := barMorphs select: [:each| each bar = anAssoc].
	m isEmpty ifTrue: [^self error: 'Bar not found'].
	barMorphs remove: m anyOne delete.
	self updateBars! !

!BarGraphMorph methodsFor: 'accessing range' stamp: 'Richo 5/21/2010 17:13'!
restoreInitialPosition
self range: (-1 to: 19); update! !

!BarGraphMorph methodsFor: 'updating' stamp: 'Richo 5/21/2010 15:56'!
updateAll
super updateAll.
self updateBars; changed! !

!BarGraphMorph methodsFor: 'updating' stamp: 'Richo 5/13/2010 20:35'!
updateBars
| index |
index := 0.
barMorphs do: [:each| each update: (index := index + 1)].
self changed! !

!CartesianGraphMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/21/2010 08:08'!
additionsToViewerCategoryPlotColors
^#(#'plot - colors' (
#(slot axisColor '' Color readWrite Player getAxisColor Player setAxisColor:  )
#(slot backgroundColor '' Color readWrite Player getBackgroundColor Player setBackgroundColor:  )
#(slot majorGridColor '' Color readWrite Player getMajorGridColor Player setMajorGridColor:  )
#(slot minorGridColor '' Color readWrite Player getMinorGridColor Player setMinorGridColor:  )
#(command restoreDefaultColors '' )
))! !

!CartesianGraphMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/21/2010 17:12'!
additionsToViewerCategoryPlotCoordinateSystem
^#(#'plot - coordinate system' (
#(command moveLeft: '' Number)
#(command moveRight: '' Number)
#(command moveUp: '' Number)
#(command moveDown: '' Number)
#(command restoreInitialPosition '')
#(slot xMax '' Number readWrite Player getXMax Player setXMax:  )
#(slot xMin '' Number readWrite Player getXMin Player setXMin:  )
#(slot yMax '' Number readWrite Player getYMax Player setYMax:  )
#(slot yMin '' Number readWrite Player getYMin Player setYMin:  )
))! !

!CartesianGraphMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/21/2010 08:08'!
additionsToViewerCategoryPlotGrid
^#(#'plot - grid' (
#(slot xMajorGrid '' Number readWrite Player getXMajorGrid Player setXMajorGrid:  )
#(slot xMinorGrid '' Number readWrite Player getXMinorGrid Player setXMinorGrid:  )
#(slot yMajorGrid '' Number readWrite Player getYMajorGrid Player setYMajorGrid:  )
#(slot yMinorGrid '' Number readWrite Player getYMinorGrid Player setYMinorGrid:  )
#(command restoreDefaultGrids '' )
))! !

!CartesianGraphMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/21/2010 08:08'!
additionsToViewerCategoryPlotLegends
^#(#'plot - legends' (
#(slot xSpacing '' Number readWrite Player getXLegendSpacing Player setXLegendSpacing:  )
#(slot ySpacing '' Number readWrite Player getYLegendSpacing Player setYLegendSpacing:  )
))! !

!CartesianGraphMorph class methodsFor: 'new-morph participation' stamp: 'Richo 5/21/2010 08:55'!
includeInNewMorphMenu
	"Return true for all classes that can be instantiated from the menu"
	^ false! !

!CartesianGraphMorph methodsFor: 'accessing colors' stamp: 'Richo 5/21/2010 07:34'!
axisColor
^axisColor! !

!CartesianGraphMorph methodsFor: 'accessing colors' stamp: 'Richo 5/21/2010 07:34'!
axisColor: aColor
axisColor := aColor.
self updateBackground.! !

!CartesianGraphMorph methodsFor: 'accessing colors' stamp: 'Richo 5/21/2010 07:35'!
backgroundColor
^backgroundColor! !

!CartesianGraphMorph methodsFor: 'accessing colors' stamp: 'Richo 5/21/2010 07:35'!
backgroundColor: aColor
backgroundColor := aColor.
self updateBackground.! !

!CartesianGraphMorph methodsFor: 'accessing cartesian bounds' stamp: 'Richo 5/21/2010 08:04'!
cartesianBounds
^cartesianBounds! !

!CartesianGraphMorph methodsFor: 'accessing cartesian bounds' stamp: 'Richo 5/21/2010 08:05'!
cartesianBounds: aRectangle
self controlCartesianBounds: aRectangle. 
cartesianBounds := aRectangle.
self update! !

!CartesianGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/21/2010 07:39'!
cartesianPointToPixel: aPoint
^(self cartesianXToPixel: aPoint x) @ (self cartesianYToPixel: aPoint y)! !

!CartesianGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/20/2010 22:06'!
cartesianPointToRelativePixel: aPoint
^(self cartesianXToRelativePixel: aPoint x) @ (self cartesianYToRelativePixel: aPoint y)! !

!CartesianGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/21/2010 07:39'!
cartesianXToPixel: aNumber
	^ (bounds left + (aNumber - cartesianBounds left * bounds width / cartesianBounds width)) rounded.
! !

!CartesianGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/20/2010 22:07'!
cartesianXToRelativePixel: aNumber

	^ (aNumber - cartesianBounds left * bounds width / cartesianBounds width) rounded.
! !

!CartesianGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/21/2010 07:39'!
cartesianYToPixel: aNumber
	^ (bounds bottom - (aNumber - cartesianBounds top * bounds height / cartesianBounds height)) rounded.
! !

!CartesianGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/20/2010 22:07'!
cartesianYToRelativePixel: aNumber
	^ (self height - (aNumber - cartesianBounds top * bounds height / cartesianBounds height)) rounded.
! !

!CartesianGraphMorph methodsFor: 'controlling errors' stamp: 'Richo 5/21/2010 08:06'!
controlCartesianBounds: aRectangle

aRectangle right <= aRectangle left ifTrue: [^self error: 'x range error'].
aRectangle bottom <= aRectangle top ifTrue: [^self error: 'y range error'].! !

!CartesianGraphMorph methodsFor: 'controlling errors' stamp: 'Richo 5/21/2010 08:06'!
controlGriddingParameter: aPoint
| |
(aPoint x isInteger and: [aPoint y isInteger]) ifFalse: [^self error: 'Gridding parameters must be natural numbers'].
(aPoint x < 0 or: [aPoint y < 0]) ifTrue: [^self error: 'Gridding parameters must be natural numbers'].! !

!CartesianGraphMorph methodsFor: 'controlling errors' stamp: 'Richo 5/21/2010 08:06'!
controlLegendSpacing: aPoint
| |
(aPoint x isInteger and: [aPoint y isInteger]) ifFalse: [^self error: 'Legend spacing must be natural numbers'].
(aPoint x < 0 or: [aPoint y < 0]) ifTrue: [^self error: 'Legend spacing must be natural numbers'].! !

!CartesianGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/21/2010 08:07'!
delete
self submorphsDo: [:each| each delete].
super delete! !

!CartesianGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 07:23'!
drawGridForm
| aCanvas |
"For performance, I keep the background in a separate form and redraw it only when it's necessary (i.e. changing the gridding parameters or the extent of the morph)"
gridForm isNil ifTrue: [^self].
aCanvas := gridForm getCanvas.
aCanvas fillColor: backgroundColor .
self drawVerticalGridOn: aCanvas; drawHorizontalGridOn: aCanvas! !

!CartesianGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 07:15'!
drawHorizontalGridOn: aCanvas
cartesianBounds top ceiling to: cartesianBounds bottom floor do: [:y | 
		| gridPos |
		gridPos := self cartesianYToRelativePixel: y.
		(y isDivisibleBy: minorGrid y) 
			ifTrue: [self drawMinorGridFrom: 0 @ gridPos to: (gridForm width - 1) @ gridPos on: aCanvas].
		(y isDivisibleBy: majorGrid y) 
			ifTrue: [self drawMajorGridFrom: 0 @ gridPos to: (gridForm width - 1) @ gridPos on: aCanvas].
		].! !

!CartesianGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 07:20'!
drawMajorGridFrom: p1 to: p2 on: aCanvas.
	((gridForm boundingBox containsPoint: p1) and: [gridForm boundingBox containsPoint: p2])
		ifFalse: [^self].
	aCanvas
				line: p1
				to: p2
				width: 1
				color: majorGridColor 
				stepWidth: 2
				secondWidth: 1
				secondColor: backgroundColor
				secondStepWidth: 2! !

!CartesianGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 07:20'!
drawMinorGridFrom: p1 to: p2 on: aCanvas

	((gridForm boundingBox containsPoint: p1) and: [gridForm boundingBox containsPoint: p2])
		ifFalse: [^self].
	aCanvas
				line: p1
				to: p2
				width: 1
				color: minorGridColor 
				stepWidth: 1
				secondWidth: 1
				secondColor: backgroundColor
				secondStepWidth: 6! !

!CartesianGraphMorph methodsFor: 'drawing' stamp: 'Richo 5/21/2010 08:29'!
drawOn: aCanvas
super drawOn: aCanvas.
aCanvas drawImage: gridForm at: bounds topLeft.
aCanvas frameRectangle: bounds color: self borderColor.! !

!CartesianGraphMorph methodsFor: 'drawing' stamp: 'Richo 5/21/2010 09:03'!
drawSubmorphsOn: aCanvas 
	"Display submorphs back to front"

	| drawBlock |
	submorphs isEmpty ifTrue: [^self].
	drawBlock := [:canvas | submorphs reverseDo: [:m | canvas fullDrawMorph: m]].
	self clipSubmorphs 
		ifTrue: [aCanvas clipBy: self clippingBounds during: drawBlock]
		ifFalse: [drawBlock value: aCanvas]! !

!CartesianGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 07:16'!
drawVerticalGridOn: aCanvas
cartesianBounds left ceiling to: cartesianBounds right floor do: [:x | 
		| gridPos |
		gridPos := self cartesianXToRelativePixel: x.
		(x isDivisibleBy: minorGrid x)
			ifTrue: [self drawMinorGridFrom: gridPos @ (gridForm height - 1) to: gridPos @ 0 on: aCanvas].
		(x isDivisibleBy: majorGrid x)
			ifTrue: [self drawMajorGridFrom: gridPos @ (gridForm height - 1) to: gridPos @ 0 on: aCanvas].
		].! !

!CartesianGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/21/2010 08:31'!
extent: newExtent
"I resize all forms to the newExtent, but (since this method is called in #initialize) I check that backbroundForm was properly initialized before, otherwise it throws an error"
super extent: newExtent.
gridForm isNil ifTrue: [^self].
self initializeGridForm; update! !

!CartesianGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/21/2010 07:33'!
initialize
super initialize.
self  clipSubmorphs: true; extent: 600@600; color: Color white; borderColor: Color gray;
	initializeCartesianBounds;
	initializeLegendSpacing;
	initializeColors;
	initializeAxes;
	initializeGrids;
	initializeLegends! !

!CartesianGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/21/2010 07:46'!
initializeAxes
	verticalAxis := PolygonMorph arrowPrototype arrowSpec: 8 @ 4; lineBorderWidth: 1.
	horizontalAxis := PolygonMorph arrowPrototype arrowSpec: 8 @ 4; lineBorderWidth: 1.
	self addMorph: horizontalAxis;
		 addMorph: verticalAxis. 
	self updateAxes ! !

!CartesianGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/20/2010 20:40'!
initializeCartesianBounds
cartesianBounds := -1@-1 corner: 19@19.! !

!CartesianGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/20/2010 20:42'!
initializeColors
backgroundColor := Color white.
minorGridColor := Color gray.
majorGridColor := Color gray.
axisColor := Color black.
! !

!CartesianGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/21/2010 08:32'!
initializeGridForm
gridForm := Form extent: self extent depth: 32.! !

!CartesianGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/21/2010 08:31'!
initializeGrids
minorGrid := 1@1.
majorGrid := 5@5.
self initializeGridForm; drawGridForm! !

!CartesianGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/20/2010 20:42'!
initializeLegendSpacing
	legendSpacing := 5@5! !

!CartesianGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/21/2010 08:04'!
initializeLegends
	legends := Dictionary new at: #x put: OrderedCollection new;
				 at: #y put: OrderedCollection new;
				 yourself.
	self placeXLegends; placeYLegends ! !

!CartesianGraphMorph methodsFor: 'accessing legends' stamp: 'Richo 5/21/2010 08:06'!
legendSpacing
^legendSpacing! !

!CartesianGraphMorph methodsFor: 'accessing legends' stamp: 'Richo 5/21/2010 08:06'!
legendSpacing: aPoint
self controlLegendSpacing: aPoint.
legendSpacing := aPoint.
self updateBackground! !

!CartesianGraphMorph methodsFor: 'accessing grid' stamp: 'Richo 5/21/2010 08:05'!
majorGrid
^majorGrid! !

!CartesianGraphMorph methodsFor: 'accessing colors' stamp: 'Richo 5/21/2010 07:35'!
majorGridColor
^majorGridColor! !

!CartesianGraphMorph methodsFor: 'accessing colors' stamp: 'Richo 5/21/2010 07:35'!
majorGridColor: aColor
majorGridColor := aColor.
self updateBackground.! !

!CartesianGraphMorph methodsFor: 'accessing grid' stamp: 'Richo 5/21/2010 08:05'!
majorGrid: aPoint
self controlGriddingParameter: aPoint.
majorGrid := aPoint.
self updateBackground ! !

!CartesianGraphMorph methodsFor: 'accessing grid' stamp: 'Richo 5/21/2010 08:05'!
minorGrid
^minorGrid! !

!CartesianGraphMorph methodsFor: 'accessing colors' stamp: 'Richo 5/21/2010 07:35'!
minorGridColor
^minorGridColor! !

!CartesianGraphMorph methodsFor: 'accessing colors' stamp: 'Richo 5/21/2010 07:35'!
minorGridColor: aColor
minorGridColor := aColor.
self updateBackground.! !

!CartesianGraphMorph methodsFor: 'accessing grid' stamp: 'Richo 5/21/2010 08:05'!
minorGrid: aPoint
self controlGriddingParameter: aPoint.
minorGrid := aPoint.
self updateBackground! !

!CartesianGraphMorph methodsFor: 'moving' stamp: 'Richo 5/21/2010 08:07'!
move: aPoint
cartesianBounds := cartesianBounds translateBy: aPoint.
self translateTurtleTrailsBy: aPoint; update! !

!CartesianGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/21/2010 08:23'!
newPlayerInstance
^ CartesianGraphPlayer newUserInstance! !

!CartesianGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 08:03'!
placeAxes
	| axisPos |
	axisPos := self cartesianPointToPixel: 0@0.
	verticalAxis vertices at: 1 put: axisPos x @ self bottom.
	verticalAxis vertices at: 2 put: axisPos x @ self top.
	verticalAxis borderColor: axisColor.
	verticalAxis computeBounds.
	horizontalAxis vertices at: 1 put: self left @ axisPos y.
	horizontalAxis vertices at: 2 put: self right @ axisPos y.
	horizontalAxis borderColor: axisColor.
	horizontalAxis computeBounds! !

!CartesianGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 08:41'!
placeXLegends
| index legendPos legend xLegends |
index := 0.
xLegends := legends at: #x.
cartesianBounds left ceiling to: cartesianBounds right floor do: [:x | 
		| pos |
		pos := self cartesianXToPixel: x.
		(x isDivisibleBy: legendSpacing x) 
			ifTrue: [
			legendPos := (self preferredPositionForXLegend: x asString at: pos).
			legendPos notNil ifTrue: [
				legend := xLegends at: (index := index + 1) ifAbsent: [xLegends add: StringMorph new].
				self addMorph: legend.
				legend contents: x asString; position: legendPos; color: axisColor]]].
xLegends isEmpty ifFalse: [
xLegends from: index + 1 to: xLegends size do: [:each | each delete].
xLegends removeLast: (xLegends size - index)]! !

!CartesianGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 18:05'!
placeYLegends

| index yLegends legendPos legend |
index := 0.
yLegends := legends at: #y.
cartesianBounds top ceiling to: cartesianBounds bottom floor do: [:y | 
		| pos |
		pos := self cartesianYToPixel: y.
		(y ~= 0 and: [(y isDivisibleBy: legendSpacing y)])
			ifTrue: [
			legendPos := (self preferredPositionForYLegend: y asString at: pos).
			legendPos notNil ifTrue: [
				legend := yLegends at: (index := index + 1) ifAbsent: [yLegends add: StringMorph new].
				self addMorph: legend.
				legend contents: y asString; position: legendPos; color: axisColor]]].
yLegends isEmpty ifFalse: [
yLegends from: index + 1 to: yLegends size do: [:each| each delete].
yLegends removeLast: (yLegends size - index)]! !

!CartesianGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 07:44'!
preferredPositionForXLegend: legendString at: gridPos
"The preferred position is below the horizontal axis at the right of the major grid.
If nothing can be drawn return nil"
| preferredPosition font axis  stringWidth |
font := TextStyle defaultFont.
stringWidth := (font widthOfString: legendString).
axis :=  horizontalAxis center y.
preferredPosition := (gridPos @ axis) + ((stringWidth / -2) rounded @ 2).

"If the legend width doesn't fit inside the form"
((gridPos + stringWidth) > self right)
	ifTrue: [^ nil "Draw nothing"].
"If the axis is not visible"
(axis < self top or: [axis > self bottom])
	ifTrue: [^preferredPosition x @ (self bottom - font height )].
"If the axis is close to the bottom so the legend height doesn't fit inside the form"
(axis > (self bottom - font height))
	ifTrue: [^preferredPosition x @ (axis - font height)].
"If the preferred position is perfect :)"
^preferredPosition ! !

!CartesianGraphMorph methodsFor: 'building' stamp: 'Richo 5/21/2010 08:24'!
preferredPositionForYLegend: legendString at: gridPos
"The preferred position is at the left of the vertical axis and above the major grid.
If nothing can be drawn return nil"
| preferredPosition font axis |
axis :=  verticalAxis center x.
font := TextStyle defaultFont.
preferredPosition := (axis - (font widthOfString: legendString)) @ (gridPos - (font height / 2)).
"If the legend height doesn't fit inside the form"
(preferredPosition y < self top)
	ifTrue: [^ nil "Draw nothing"].
"If the axis is not visible"
(axis < self left or: [axis > self right])
	ifTrue: [^self left + 3 @ preferredPosition y].
"If the axis is close to the left so the legend width doesn't fit inside the form"
(preferredPosition x < self left)
	ifTrue: [^(axis + 3 @ preferredPosition y) ].
"If the preferred position is perfect :)"
^preferredPosition! !

!CartesianGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/21/2010 08:06'!
relativePixelToCartesianPoint: aPoint
^(self relativePixelToCartesianX: aPoint x) @ (self relativePixelToCartesianY: aPoint y)
! !

!CartesianGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/21/2010 08:06'!
relativePixelToCartesianX: anInteger
^cartesianBounds left + (anInteger * cartesianBounds width / bounds width)! !

!CartesianGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/21/2010 08:06'!
relativePixelToCartesianY: anInteger
^cartesianBounds top + ((bounds height - anInteger) * cartesianBounds height / bounds height)! !

!CartesianGraphMorph methodsFor: 'accessing colors' stamp: 'Richo 5/21/2010 07:35'!
restoreDefaultColors
self initializeColors; updateBackground! !

!CartesianGraphMorph methodsFor: 'accessing grid' stamp: 'Richo 5/21/2010 08:05'!
restoreDefaultGrids
self initializeGrids; updateBackground! !

!CartesianGraphMorph methodsFor: 'accessing cartesian bounds' stamp: 'Richo 5/21/2010 08:05'!
restoreInitialPosition
"self initializeCartesianBounds."
| originalBounds |
originalBounds := -1@-1 corner: 19@19.

self move: originalBounds center - cartesianBounds center.
self update! !

!CartesianGraphMorph methodsFor: 'moving' stamp: 'Richo 5/21/2010 08:07'!
translateTurtleTrailsBy: aPoint
| newForm |
turtleTrailsForm ifNil: [^self].

newForm := turtleTrailsForm blankCopyOf: turtleTrailsForm scaledBy: 1.
turtleTrailsForm contentsOfArea: (turtleTrailsForm boundingBox translateBy: 
(self cartesianPointToRelativePixel: aPoint + cartesianBounds bottomLeft )) into: newForm.
turtleTrailsForm := newForm.
turtlePen := Pen newOnForm: turtleTrailsForm
! !

!CartesianGraphMorph methodsFor: 'updating' stamp: 'Richo 5/21/2010 07:35'!
update
self updateAll! !

!CartesianGraphMorph methodsFor: 'updating' stamp: 'Richo 5/21/2010 08:02'!
updateAll
self updateBackground; changed! !

!CartesianGraphMorph methodsFor: 'updating' stamp: 'Richo 5/21/2010 07:48'!
updateAxes
	| axisPos |
	axisPos := self cartesianPointToPixel: 0@0.
	verticalAxis vertices at: 1 put: axisPos x @ self bottom.
	verticalAxis vertices at: 2 put: axisPos x @ self top.
	verticalAxis borderColor: axisColor.
	verticalAxis computeBounds.
	horizontalAxis vertices at: 1 put: self left @ axisPos y.
	horizontalAxis vertices at: 2 put: self right @ axisPos y.
	horizontalAxis borderColor: axisColor.
	horizontalAxis computeBounds! !

!CartesianGraphMorph methodsFor: 'updating' stamp: 'Richo 5/21/2010 08:03'!
updateBackground
self drawGridForm; placeAxes; placeXLegends; placeYLegends; changed! !

CartesianGraphMorph subclass: #DataGraphMorph
	instanceVariableNames: 'vectorMorphs pointMorphs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!DataGraphMorph commentStamp: 'Richo 5/21/2010 08:57' prior: 0!
This class is responsible of drawing points and vectors in a cartesian coordinate system.

To add points use the following protocol:
	#pointAt:
	#pointAt:put:
	#removePointAt:
	#removeAllPoints
	
To add vectors use the following protocol:
	#addVector:
	#vectorAt:
	#vectorAt:put:
	#removeVector:
	#removeVectorAt:
	#removeAllVectors
	
For more information, see the "examples" category at the class side.

!

!DataGraphMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/21/2010 09:06'!
additionsToViewerCategoryPlotPoints
^#(#'plot - points' (
#(slot pointIndex '' Number  readWrite Player getPointIndex Player setPointIndex:  )
#(command removePointAt: '' Number )
#(command removeAllPoints '')
#(command removeCurrentPoint '')
#(command addPoint '')
#(slot pointX '' Number readWrite Player getPointX Player setPointX:  )
#(slot pointY '' Number readWrite Player getPointY Player setPointY:  )
#(command openPointTable '')
#(slot pointCount '' Number readOnly Player getPointCount Player unused  )
#(slot pointColor '' Color readWrite Player getPointColor Player setPointColor:  )
#(slot playerAtPointIndex '' Player readWrite Player getPlayerAtPointIndex  unused unused)
))! !

!DataGraphMorph class methodsFor: 'viewer categories' stamp: 'Richo 5/21/2010 09:16'!
additionsToViewerCategoryPlotVectors
^#(#'plot - vectors' (
#(slot vectorIndex '' Number  readWrite Player getVectorIndex Player setVectorIndex:  )
#(command removeVectorAt: '' Number )
#(command removeAllVectors '')
#(command removeCurrentVector '')
#(command addVector '')
#(slot initialPointX '' Number readWrite Player getInitialPointX Player setInitialPointX:  )
#(slot initialPointY '' Number readWrite Player getInitialPointY Player setInitialPointY:  )
#(slot terminalPointX '' Number readWrite Player getTerminalPointX Player setTerminalPointX:  )
#(slot terminalPointY '' Number readWrite Player getTerminalPointY Player setTerminalPointY:  )
#(slot vectorMagnitude '' Number readWrite Player getVectorMagnitude Player setVectorMagnitude:  )
#(slot vectorAngle '' Number readWrite Player getVectorAngle Player setVectorAngle:  )
#(slot vectorCount '' Number readOnly Player getVectorCount Player unused  )
#(slot vectorColor '' Color readWrite Player getVectorColor Player setVectorColor:  )
#(slot playerAtVectorIndex '' Player readWrite Player getPlayerAtVectorIndex  unused unused)
))! !

!DataGraphMorph class methodsFor: 'parts bin' stamp: 'Richo 4/15/2010 12:36'!
defaultNameStemForInstances
	^ 'Graph' ! !

!DataGraphMorph class methodsFor: 'parts bin' stamp: 'Richo 4/15/2010 15:40'!
descriptionForPartsBin
	^ self
		partName: 'Graph'
		categories: #('GSoC' )
		documentation: ''
! !

!DataGraphMorph class methodsFor: 'examples' stamp: 'Richo 5/21/2010 08:55'!
examplePoints
"To watch the example simply execute (Cmd-d):

self examplePoints

"
| graph | 

"Creating the morph and opening it in the world"
graph := DataGraphMorph new.
graph openInHand.

"Setting the grid parameters"
graph minorGrid: 2@3.
graph majorGrid: 4@6.

"Setting the legend spacing"
graph legendSpacing: 8@6.

"Setting the x/y range"
graph cartesianBounds: ((-15@-15) corner: (15@15)).

"Changing the default colors"
graph backgroundColor: Color black.
graph minorGridColor: Color cyan.
graph majorGridColor: Color magenta.
graph axisColor: Color white.

"Ploting a sine wave"
-15 to: 15 by: 0.5 do: [:x|
	graph addPoint: x @ (x sin).
].! !

!DataGraphMorph class methodsFor: 'examples' stamp: 'Richo 5/21/2010 08:45'!
exampleVectors
"To watch the example simply execute (Cmd-d):

self exampleVectors

"
| graph | 

"Creating the morph and opening it in the world"
graph := DataGraphMorph new.
graph openInHand.

"Changing the minor grid"
graph minorGrid: 0@0.

"Adding a vector"
graph addVector: (Vector terminalPoint: 10@5).
graph vectorColorAt: 1 put: Color red muchDarker.

"Adding another vector"
graph addVector: (Vector terminalPoint: 3@10).
graph vectorColorAt: 2 put: Color red.

"Adding the sum of the vectors above"
graph addVector: (Vector terminalPoint: 13@15).
graph vectorColorAt: 3 put: Color red muchLighter.

"Adding two additional vectors"
graph addVector: ((3@10) to: (13@15)).
graph vectorColorAt: 4 put: Color red muchDarker.
graph addVector: ((10@5) to: (13@15)).
graph vectorColorAt: 5 put: Color red.! !

!DataGraphMorph class methodsFor: 'new-morph participation' stamp: 'Richo 4/23/2010 17:30'!
includeInNewMorphMenu
	"Return true for all classes that can be instantiated from the menu"
	^ true! !

!DataGraphMorph methodsFor: 'menus' stamp: 'Richo 4/26/2010 22:37'!
addCustomMenuItems: aMenu hand: aHandMorph
aMenu add: 'open point table' target: self action: #openPointTable! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/25/2010 15:09'!
addPoint: aPoint
| m |
m := PointMorph graph: self point: aPoint.
self addMorph: m.
pointMorphs add: m.! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/20/2010 14:50'!
addVector: aVector
| m |
m := VectorMorph graph: self vector: aVector.
self addMorph: m.
vectorMorphs add: m.
! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/17/2010 00:37'!
indexOfVector: aVector
| index |
index := 0.
vectorMorphs do: [:each|
	index := index + 1.
	each vector = aVector
	ifTrue: [
			^index
		]
	].
^0! !

!DataGraphMorph methodsFor: 'initialize' stamp: 'Richo 5/21/2010 08:46'!
initialize
super initialize.
self
	initializePoints;
	initializeVectors.
! !

!DataGraphMorph methodsFor: 'initialize' stamp: 'Richo 4/20/2010 23:50'!
initializePoints
pointMorphs := OrderedCollection new! !

!DataGraphMorph methodsFor: 'initialize' stamp: 'Richo 4/15/2010 15:09'!
initializeToStandAlone
	"Answer an instance of the receiver suitable for placing in a parts bin for authors"
	
	self initialize.! !

!DataGraphMorph methodsFor: 'initialize' stamp: 'Richo 4/16/2010 22:04'!
initializeVectors
vectorMorphs := OrderedCollection new ! !

!DataGraphMorph methodsFor: 'initialize' stamp: 'Richo 4/16/2010 21:32'!
newPlayerInstance
^ DataGraphPlayer newUserInstance! !

!DataGraphMorph methodsFor: 'actions' stamp: 'Richo 4/15/2010 15:44'!
openPointTable
(PointTableMorph on: self) openInHand! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/20/2010 23:17'!
pointAt: index
^(pointMorphs at: index) point! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/20/2010 23:23'!
pointAt: index ifAbsent: aBlock
| m |
m := pointMorphs at: index ifAbsent: [nil].
^m isNil ifTrue: [aBlock value] ifFalse: [m point]
! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/20/2010 23:21'!
pointAt: index put: aPoint
(pointMorphs at: index ifAbsent: [^self error: 'Point not found']) point: aPoint.! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/20/2010 23:43'!
pointColorAt: index
^(pointMorphs at: index) color! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/20/2010 23:44'!
pointColorAt: index put: aColor
(pointMorphs at: index) color: aColor! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/20/2010 23:40'!
pointCount
^pointMorphs size! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 5/21/2010 09:12'!
pointMorphAt: index
^pointMorphs at: index ifAbsent: [nil]! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/20/2010 23:57'!
points
^pointMorphs collect: [:each| each point]! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/20/2010 23:57'!
removeAllPoints
	pointMorphs do: [:each| each delete].
	self initializePoints	! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/16/2010 22:03'!
removeAllVectors
	vectorMorphs do: [:each| each delete].
	self initializeVectors! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/20/2010 23:56'!
removePointAt: index
	| p |
	p := pointMorphs at: index ifAbsent: [^self error: 'Point not found'].
	p delete.
	pointMorphs removeAt: index.! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/27/2010 12:34'!
removePointsFrom: start to: stop
| removed |
removed := OrderedCollection new.
start to: stop do: [:i|
	removed add: (pointMorphs at: i) delete.
	].
pointMorphs removeAllFoundIn: removed! !

!DataGraphMorph methodsFor: 'accessing points' stamp: 'Richo 4/20/2010 23:16'!
removePoint: aPoint
	| p |
	p := pointMorphs select: [:each| each point = aPoint].
	p isEmpty ifTrue: [^self error: 'Point not found'].
	pointMorphs remove: p anyOne delete.! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/17/2010 00:49'!
removeVectorAt: index
	| m |
	m := vectorMorphs at: index ifAbsent: [^self error: 'Vector not found'].
	m delete.
	vectorMorphs removeAt: index.
! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/17/2010 00:48'!
removeVector: aVector 
	| v |
	v := vectorMorphs select: [:each| each vector = aVector].
	v isEmpty ifTrue: [^self error: 'Vector not found'].
	vectorMorphs remove: v anyOne delete.! !

!DataGraphMorph methodsFor: 'updating' stamp: 'Richo 5/21/2010 08:47'!
updateAll
super updateAll.
self updatePoints; updateVectors; changed! !

!DataGraphMorph methodsFor: 'updating' stamp: 'Richo 4/20/2010 23:50'!
updatePoints
	"Updates the position of each point"
	pointMorphs do: [:each | each update].
	self changed! !

!DataGraphMorph methodsFor: 'updating' stamp: 'Richo 4/20/2010 23:22'!
updateVectors
	"Updates the position of each vector"
	vectorMorphs do: [:each | each update].
	self changed! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/16/2010 22:03'!
vectorAt: index
^(vectorMorphs at: index) vector! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/20/2010 23:23'!
vectorAt: index ifAbsent: aBlock
| m |
m := vectorMorphs at: index ifAbsent: [nil].
^m isNil ifTrue: [aBlock value] ifFalse: [m vector]
! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/20/2010 23:18'!
vectorAt: index put: aVector
(vectorMorphs at: index ifAbsent: [^self error: 'Vector not found']) vector: aVector.! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/16/2010 22:45'!
vectorColorAt: index
^(vectorMorphs at: index) color! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/16/2010 22:41'!
vectorColorAt: index put: aColor
(vectorMorphs at: index) color: aColor! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 4/16/2010 22:03'!
vectorCount
	^vectorMorphs size! !

!DataGraphMorph methodsFor: 'accessing vectors' stamp: 'Richo 5/21/2010 09:16'!
vectorMorphAt: index
^vectorMorphs at: index ifAbsent: [nil]! !

PolygonMorph subclass: #VectorMorph
	instanceVariableNames: 'graph vector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSoC-Graphing'!

!VectorMorph commentStamp: 'Richo 4/18/2010 00:23' prior: 0!
This object is used by DataGraphMorph to draw vectors. It needs to know the graph in which it is plotted to be able to update its position whenever necessary.!

!VectorMorph class methodsFor: 'viewer categories' stamp: 'Richo 4/17/2010 00:23'!
additionsToViewerCategories
^ #(
(#plot (
#(command remove '' )
#(slot initialPointX '' Number readWrite Player getInitialPointX Player setInitialPointX:  )
#(slot initialPointY '' Number readWrite Player getInitialPointY Player setInitialPointY:  )
#(slot terminalPointX '' Number readWrite Player getTerminalPointX Player setTerminalPointX:  )
#(slot terminalPointY '' Number readWrite Player getTerminalPointY Player setTerminalPointY:  )
#(slot vectorAngle '' Number readWrite Player getVectorAngle Player setVectorAngle:  )
#(slot vectorColor '' Color readWrite Player getVectorColor Player setVectorColor:  )
#(slot vectorMagnitude '' Number readWrite Player getVectorMagnitude Player setVectorMagnitude:  )
))

)! !

!VectorMorph class methodsFor: 'instance creation' stamp: 'Richo 4/16/2010 18:07'!
graph: aDataGraphMorph vector: aVector
^(self basicNew setGraph: aDataGraphMorph vector: aVector) initialize! !

!VectorMorph methodsFor: 'accessing' stamp: 'Richo 4/16/2010 22:45'!
color
	^borderColor! !

!VectorMorph methodsFor: 'accessing' stamp: 'Richo 4/16/2010 12:26'!
color: aColor
	self borderColor: aColor.
	super color: aColor.! !

!VectorMorph methodsFor: 'initialize-release' stamp: 'Richo 4/27/2010 14:47'!
dismissViaHalo
	self remove.
	super dismissViaHalo! !

!VectorMorph methodsFor: 'accessing' stamp: 'Richo 4/16/2010 12:21'!
from: startPoint to: endPoint
	"We don't calculate clipped vector, otherwise the arrow may be misplaced when part of the vector extremity is outisde the drawing area'"
	vertices at: 1 put: startPoint asIntegerPoint .
	vertices at: 2 put: endPoint  asIntegerPoint.
	self computeBounds
! !

!VectorMorph methodsFor: 'initialize-release' stamp: 'Richo 4/17/2010 01:07'!
initialize
super initialize.
self makeOpen.
vertices := Array with: 0@0 with: 0@0.
arrows := #forward.
self arrowSpec: 4@3; color: Color blue.
self update! !

!VectorMorph methodsFor: 'initialize-release' stamp: 'Richo 4/17/2010 00:23'!
newPlayerInstance
^ VectorPlayer newUserInstance! !

!VectorMorph methodsFor: 'accessing' stamp: 'Richo 4/27/2010 15:23'!
position: aPoint
super position: aPoint.
self update! !

!VectorMorph methodsFor: 'private' stamp: 'Richo 4/27/2010 15:22'!
privateCenter: aPoint
self privatePosition: (aPoint - (self extent // 2))! !

!VectorMorph methodsFor: 'private' stamp: 'Richo 4/27/2010 15:22'!
privatePosition: aPoint
^super position: aPoint! !

!VectorMorph methodsFor: 'initialize-release' stamp: 'Richo 4/17/2010 00:45'!
remove
	graph removeVector: vector! !

!VectorMorph methodsFor: 'private' stamp: 'Richo 4/16/2010 18:13'!
setGraph: aDataGraphMorph vector: aVector
graph := aDataGraphMorph.
vector := aVector! !

!VectorMorph methodsFor: 'update' stamp: 'Richo 4/23/2010 16:08'!
update
self penUpWhile: [self vector: vector]! !

!VectorMorph methodsFor: 'accessing' stamp: 'Richo 4/16/2010 18:06'!
vector
^vector! !

!VectorMorph methodsFor: 'accessing' stamp: 'Richo 4/23/2010 16:05'!
vector: aVector
self from: (graph cartesianPointToPixel: aVector initialPoint) to: (graph cartesianPointToPixel: aVector terminalPoint).
vector := aVector! !
BarGraphMorph initialize!
